
(*
:Mathematica Version: 4.0
*)
(*
:Package Version: 1.1
*)
(*
:Copyright: public domain
*)
(*
:Name: cr`
*)
(*
:Title: n chinese remainders
*)
(*
:Author: John Hendrickson (debguy@sourceforge.net)
*)
(*
:History: .1 minor packaging improvement
*)
(*
:Keywords:
        number theory, Chinese Remainder Theorem,
        euclalg, euclalglist, euclbacksolve, euclextgcd, inverseMod,
        testgen, showFactors, changeBaseAArr, caesar
        crtwo, cralign1, align1,
        crchart3, creqs, crfindnSolnAfterxInEqm
        crAll, crAll2, IntToModTups,
        congQ, congX, crbruteforce, crtestnotcong, crtestcong
*)
(*
:Requirements: None
*)
(*
:Sources: all code is origional by author
*)
(*
:Summary:

	Presenting:

	crtwo: solves Chinese Remainder by simple pair at a time
		crAll2@crtwo provides n soln near x for CR, uses cr width
		crchart3 shows soln as sawtooth waves (CR) meeting
	align1: solves i m2 == j m2 + b -> {i,j,dist}
		like PowerMod->i but b is any and gcd(m1,m2) not req.
	other funs:
		Euclidians, showFactors, changeBaseArr, caesar, rsa, 
		inverseMod, IntToModTups (large number math for PC's)
		crfindnSolnAfterxInEqm (solve saw wave LL,RL,LR,... intersects)
		congruence, plots, y-shifted triangular wave solve, testing

	by: John Hendrickson (debguy@sourceforge.net), 2013, public domain
		credit to many authors of math in general

	partial book notes were from:
	Rosen, Kenneth H., "Discrete Mathematics and it's Applications", 3rd ed.

see divs.nb (n-chinese-remainders.nb) for use, plots and discussions, and a \
few extra funs
*)
(*
setup
*)
(* suggested , see the mm book *)
If[False,
  Off[General::"spell"];(* turn off has similar spelling messages *)
  (* 
    On[MemoryConserve] *)(* may be causing crashes *)
  Unprotect[In, Out];
  Clear[In, Out];
  Protect[In, Out];
  $HistoryLength = 5;
  MemoryInUse[];
  ]
Needs["NumberTheory`NumberTheoryFunctions`"];
BeginPackage["cr`", {"NumberTheory`NumberTheoryFunctions`"}];
Off[General::"spell"] (* turn off has similar spelling messages *)
Clear[
  k, rand, testgen,
  Mod1, Mod2, Mod3, Mod4, Mod8, Mod5,
euclalg, euclalg2, euclalglist, euclbacksolve, euclextgcd, \
euclalglistUnsigned, euclextgcd,
showFactors,
changeBaseArr,
caesar,
congQ, congQM, congQM1, congQMboth, congX, congA,
crbruteforce, crtestcong, crtestnotcong,
inverseMod, inverseMod2,
crchart3,
combpairs, creqs, crxtotriag, crtriagtox,
crfindnSolnAfterxInEqm,
cralign1, testcra1, align1, testa1,
crtwo, crtwo2, chineseremainder, crtlist,
crAll, crAll2,
IntToModTups, AddModTups, SubModTups, MulModTups, TupsModToInt
  ]
cr::"usage" = 
    "cr` is context for n-chinese-remainders (crtwo) keeps symbols.  see";
Mod1::usage = "Mod1-8 are for study purpose";
euclalg::usage = "euclalg return should be same as GCD";
euclalglist::usage = 
    "euclalglist returns {r,b,q} meaning \
{{a1,b1,0},{r1,b1,q1},{r2,r1,q2},...} for a=q*b+r each successive euclalg";
euclextgcd::usage = "euclextgcd return should be same as ExtendedGCD";
showFactors::usage = "showFactors[n]  a product of primes display of n";
changeBaseArr::usage = 
    "changeBaseArr[n,b]  shows list of digits of number n in base b, the list \
is given in base 10";
caesar::usage = 
    "caesar[msg,encode] cipher and de-cipher (enc False). for study purpose";
congQ::usage = 
    "congQ[a,b,m] , m|(a-b) , congruentQ , a \[Congruent] Mod[b,m] , b \
\[Congruent] Mod[a,m] , a == b + k m , a or b can contain k (ie 4+7k) if m | \
(a - b) or only k is left or result is 0 (a==b).  if congQ returns \
\"expr==0\" for (ie a==4+7k) expr is not always cong but particular choices \
of k may cause r==0 or m|(a-b), which is neither T or F.  In general use \
congQ==True since \[Not]congQ is T when congQ did nothing.  for N@a consider \
using Rationalize@a.  see congQM";
congQM::usage = 
    "congQM[a,b,m] checks congruence by Mod[a,m]==Mod[b,m].  note for neg num \
Mod assumes a,b,m work by Floor not IntegerPart (Mathematica a/m=q+r/m funs \
use Floor, which is ok if a,b,m are found using mm funs), see Mod details, \
congQM1, congQMboth";
congX::usage = 
    "congX[b,a,m,k:k] (b-a k)=m c , return eq or value k , Mod[a k,m] \
\[Congruent] Mod[b,m] , congQ[a congX[b,a,m,k?],b,m] , for any choice k.  \
default for k is symbol k (subst after).  see also congA.  a right choice of \
k is inv if there is one, see inverseMod.";
congA::usage = 
    "congA[b,m,k:k]  .5 broken.  return eq or value a k , a \[Congruent] \
Mod[b k, m] , congQ[congA[b,m],b,m] , for any choice k.  default for k is \
symbol k (subst after).  see also congX. use congX if b==0 then GCD[a,m]==1 \
needs to be tested. does not work with fractions.  not suggested";
inverseMod::usage = "inverseMod[b,[-1],m]  \!\(a\^\(-1\)\) \[Congruent] \
Mod[x,m] ; a x == a \!\(TraditionalForm\`a\^\(-1\)\)\[Congruent] Mod[1,m] /; \
m!=0 && .  see also congX";
crchart3::usage = 
    "crchart3[bi,mi,xi:0,xf:30,show:True]  shows sawtooth waves that \
represent the chinese remainder theorem moduli (each wave is one pair). \
moduli have soln where they cross x axis and where all lefts on axis meet a \
soln lies.";
creqs::usage = 
    "creqs[bi_:{2,5,6},mi_:{3,7,10},eq]  gives eqs for solving chinese \
remainder theorem that show where LL triangles meet axis, or any L or R pairs \
to meet (see crchart3). for Length 3 there are 2 2 2 eqs (m can be All though \
only m1's are cr soln).  eq==False is none, and 1 means only 1st, True is \
all.  {{bi,mi,r,h},{leq,req,sr,eqs}}.  see crchart3, crfindnSolnAfterxInEqm";
creqs::lengths = "bi mi are different lengths";
crfindnSolnAfterxInEqm::usage = 
    "crfindnsolnaftertriagineqm[bi,mi,n:3,x:0,lim,m:1,modchk,dbg]  solves \
chinese remainder theorem by brute force per say.  however cr finds only Left \
L (see crchart3) of triangles, this fun can find L or R for each triag to \
each other L or R and can work with fractions.  caveats: likely slow if mi > \
10 items.  \"find the next n soln of eq m of given x offset\". lim is how \
many loops to try.  modchk True uses Mod, takes longer, but works with \
fractions (forced if not Integer).  m can be All (1 is cr, see creqs, if m!=1 \
there are 2^lenmi small expr, allot). see crchart3, crAll, crtwo.";
crfindnSolnAfterxInEqm::"noeqn" = "creqs eq has only `1` eq";
crfindnSolnAfterxInEqm::"nosolve" = "warn: creqs eq `1` does not solve";
crfindnSolnAfterxInEqm::"exhausted" = 
    "exhausted searching , none found so far";
align1::usage = 
    "align1[sawm1,offset,sawm2,bothoff,dbg]  solves m2 i == m1 j + offset, \
and returns {i,j,dist} and all are Integers, a useful extention to \
PowerMod(m2 i == m1 j + 1, returns i).  thinking of LCM[mi] as tot width to \
sawtooth waves meeting from one intersection to next and dist as meeting when \
offset is involved.  soln can be same as chinese remainder. (meaning sawtooth \
with x and y offsets that meet x axis)(PM funs are essential when working \
with math where solutions must also be Integer)  see also: crchart3, \
cralign1, align1non0";
cralign1::usage = 
    "cralign1[sawm1,offset,sawm2,bothoff,dbg]  solves m2 i == m1 j + offset, \
and returns dist. thinking of LCM[mi] tot width to sawtooth waves: what about \
when the starting point is not aligned?  soln can be same as chinese \
remainder, dist.  for cralign j can be 0 and i,j,dist are Integers. \
(sawtooths having y offset known by x offset tooth widths. for plain \
sawtooths LCM@@mi is ans)  see also: crchart3, crtwo, crAll, crbruteforce \
align1";
crtwo::usage = 
    "crtwo[bi,mi,dbg]  solves chinese remainder theorem, the sys of eq  x mod \
m \[Congruent] b mod m (lists b m, x scalar) and returns {x intersect, width \
to next}  benefits: x is always positive. gives width (see crAll). it quits \
much sooner when there is no soln (pair at a time)(dbg shows first \
non-congruent).  uses no memory except the input (CR creates lists).  comes \
with useful sawtooth & multiples aligners.  caveates: usu. takes twice as \
long or for large data (but is bigO linear), though beats CR on a few.  crtwo \
is faster on the average if more than 1/2 of solving attempts will not find a \
soln.  notes: CR and crtwo work only on Integer and give wrong answers for \
fractions (so multiply first) (all Rs of sawtooths align at 0: but CR can \
give negative ans).  see also: crchart3, cralign1, crAll2, crbruteforce";
crtwo::"lengths" = "bi mi are different lengths";
crAll::usage = 
    "crAll[bi,mi,n:5,x:0,m]  solves chinese remainder theorem, the sys of eq  \
x \[Congruent] b mod m (b and m are lists)  find the next n consecutive soln \
given x which is where to start looking (can be neg).  note soln may include \
x +- smallest value of modulus (smallest triangle size) that's chinese \
remainder soln.  crAll2 needs {intersect, width} instead of bi,mi.  see also: \
crchart3, crtwo, crbruteforce, crfindnSolnAfterxInEqm (finds first solns)";
IntToModTups::usage = 
    "IntToModTups[n,v]  n is converted by v, and after add sub mul, is \
converted back by TupsModToInt.  not well tested, for discussion purposes.  \
(to choose non-default v, m = m1 m2 m3 ... where mk > 2 and gcd(mi,mj)==1 \
when i!=j.  note TMTI uses CR not crtwo, whether nearest cr after x==0 or \
signed cr is right: is left in the air";
k =.; (* congX needs to share k, not in a context yet, 
  so Clear do not Remove *)
(*
rand[] := ((Random[] 20) - 10)
*)
(*
mm  chineseremainder  and  crtlist
*)
(*
mm 4.0 contains ChineseRemainder and CRTlist in.  crlist is normally tucked \
in private.  see  \
mathematica/AddOns/StandardPackages/NumberTheory/NumberTheoryFunctions.m
*)
(*
chineseremainder = ChineseRemainder;
*)
crtlist = NumberTheory`NumberTheoryFunctions`Private`CRTlist;
Begin["`Private`"];
(*
testgen
*)
(*
	generic brute force test function
*)
rand[] := ((Random[] 20) - 10)
testgen[f1_, f2_, n_:100, dbg_:False] := 
  Module[{a, b, m, last, next, rR, rI},
    Do[
      a = rand[]; b = rand[]; m = rand[];
      a = Rationalize[a, .01];
      b = Rationalize[b, .01];
      m = Rationalize[m, .01];
      If[f1[a, b, m] != f2[a, b, m], 
        Print@{a, b, m, f1[a, b, m], f2[a, b, m]}];
      a = Floor@a; b = Floor@b; m = Floor@m;
      If[f1[a, b, m] != f2[a, b, m], 
        Print@{a, b, m, f1[a, b, m], f2[a, b, m]}];
      , {n}]
    ]
testgen[f1_, f2_, n_:100, ruleR_, ruleI_, dbg_:False] := 
  Module[{a, b, m, last, next, rR, rI},
    Do[
      a = rand[]; b = rand[]; m = rand[];
      a = Rationalize[a, .01];
      b = Rationalize[b, .01];
      m = Rationalize[m, .01];
      {as, bs, ms} = Abs@{a, b, m};
      If[ruleR[a, b, m], 
        If[f1[a, b, m] != f2[a, b, m], 
          Print@{a, b, m, f1[a, b, m], f2[a, b, m]}]];
      a = Floor@a; b = Floor@b; m = Floor@m;
      {as, bs, ms} = Abs@{a, b, m};
      If[ruleI[a, b, m], 
        If[f1[a, b, m] != f2[a, b, m], 
          Print@{a, b, m, f1[a, b, m], f2[a, b, m]}]];
      , {n}]
    ]
testgen[f1_, f2_, n_:100, ruleR_, ruleI_, dbg_:False] := 
  Module[{a, b, m, last, next, rR, rI},
    Do[
      a = rand[]; b = rand[]; m = rand[];
      a = Rationalize[a, .01];
      b = Rationalize[b, .01];
      m = Rationalize[m, .01];
      {as, bs, ms} = Abs@{a, b, m};
      If[ruleR[a, b, m], 
        If[f1[a, b, m] != f2[a, b, m], 
          Print@{"hi", a, b, m, f1[a, b, m], f2[a, b, m]}]];
      a = Floor@a; b = Floor@b; m = Floor@m;
      {as, bs, ms} = Abs@{a, b, m};
      If[ruleI[a, b, m], 
        If[f1[a, b, m] != f2[a, b, m], 
          Print@{a, b, m, f1[a, b, m], f2[a, b, m]}]];
      , {n}]
    ]
(*
Mod 1-5,8
*)
(*
	Mod1[b,m] shows that for neg nums there are two right answers.
		the difference is whether a/m=q+r/m whether
		q and r use Floor or IntegerPart (q and r must use same)
	The book and mathematica choose positive remainders, Mod1 chooses other.
		n Quotient[m, n, d] + Mod[m, n, d] == m
*)
Mod1[a_, m_] := a - m IntegerPart[a/m]
Mod2[a_, m_] := 
  m/2 - (m/Pi) ArcTan[Cot[(Pi a)/m]] /; 
    a/m \[Element] Reals && ! a/m \[Element] Integers
(*
	note Mod2 and Mod4 do not, in general, give all soln
*)
Mod2[a_, m_] := 0 /; m != 0 && a/m \[Element] Integers
\!\(Mod3[a_, m_] := a - m\ Floor[a\/m]\)
Mod4[a_, m_] := 
  Last@IntegerDigits[a, m] /; 
    a \[Element] Integers && m \[Element] Integers && m > 1 && a > 1
Mod4[a_, m_] := a - m Quotient[a, m]
Mod8[a_, m_] := a - m Quotient[a, m]
SetAttributes[Mod5, {Listable, NumericFunction}];
(*
purpose of Mod5 is to show Sign needs and result needs. very slow if b << a \
(by more than a few powers of 10)  	(Mod5 doesn't approx, does do frac)
*)
Mod5[a_, m_] := Module[{x, y, ms},
    If[m == 0, Return[]]; If[a == 0, Return[0]];
    If[Sign@m == 1, If[Sign@a == 1,
        If[a < m, Return[a]];
        y = (a - m); While[y >= m, y -= m]; Return[y]
        , y = (m + a); While[y < m, y += m]; Return[y - m]]
      , If[Sign@a == 1,
        y = (m + a); While[y > m, y += m]; Return[y - m]
        , If[a > m, Return[a]];
        y = (a - m); While[y <= m, y -= m]; Return[y]
        ]]]
(*
Euclidian Algorithms  gcd  gcdlist  extended gcd
*)
(*
simple gcd and extended gcd (pairs, no List)
*)
(*
divides until least remainder is found for a1,a2
	a1 = q1 * b1 + r1
	b1 = q2 * r1 + r2
	r1 = q3 * r2 + r3 ...
*)
(*
euclalg[m2_,m1_]:=(If[m2==0,Return[m1]];euclalg[m1-m2 Quotient[m1,m2],m2])
*)
euclalg[m2_, m1_] := 
  Module[{x, r, q}, q = Abs@m1; r = Abs@m2; If[q < r, x = q; q = r; r = x]; 
    If[r == 0, Return@q]; 
    While[True, x = Mod[q, r]; If[x == 0, Return[r]]; q = r; r = x]]
euclalg2[{m2_, m1_, c_}] := (If[m2 == 0, Return[{0, 0, 0}]]; {Mod[m1, m2], m2,
       Quotient[m1, m2]})
(*
	return is sequence of {r,b,q} ; a==m1, b==m2, a == q * b + r
*)
euclalglist[pm2_, pm1_] := Module[{m2, m1, x},
    If[euclalg2[{0, 0, 0}] != {0, 0, 0}, Print@"err"; Return[]];
    m1 = Abs@pm1; m2 = Abs@pm2; If[m1 < m2, x = m1; m1 = m2; m2 = x]; 
    If[m2 == 0, Return@m1];
    NestWhileList[euclalg2, {m2, m1, 0}, (First@#1 != 0) &]
    ]
euclbacksolve[last_, this_, n_, v_] := Module[{},
    If[n >= Length@v, Return[{last, this}]];
    euclbacksolve[this, last - this v[[n + 1]], n + 1, v]
    ]
(*
v=Table[b[i],{i,1,5}];
euclbacksolve[1,-v[[3]],3,v]
should do this:
     {1+b[3] b[4],-b[3]-(1+b[3] b[4]) b[5]}
*)
euclextgcd[pm2_Integer, pm1_Integer] := 
  Module[{i, j, x, m2, m1, sm2, sm1, signdiff, m2more, v, gcd},
    If[pm1 == 0, Return[{Abs@pm2, {Sign@pm2, 0}}]];
    If[pm2 == 0, Return[{Abs@pm1, {0, Sign@pm1}}]];
    (* do work easy w/o signs *)
    sm1 = Sign@pm1; sm2 = Sign@pm2;
    If[Abs@pm2 > Abs@pm1, m2more = True; m1 = Abs@pm2; m2 = Abs@pm1, 
      m2more = False; m1 = Abs@pm1; m2 = Abs@pm2];
    v = euclalglist[m2, m1]; gcd = (Last@v)[[2]];
    v = v[[All, 3]]; v = Reverse@v; v = {0}~Join~v;
    v = euclbacksolve[1, -v[[3]], 3, v];
    (* replace signs. see note *)
    If[ListQ@v && NumberQ@First@v,
      If[m1 === m2,
        v = Abs@v{sm1, -sm2}; v = Reverse@v
        , If[m2more, v = Reverse@v];
        If[sm1 == -1, v[[2]] *= -1]; If[sm2 == -1, v[[1]] *= -1];
        ]];
    Return@{gcd, v}
    ]
(*
luckily mm has convention; +gcd(a,b) = sa + tb /;gcd>0  (lucky since matching \
signs is tedious i'm sure).  for m1==m2 just cp what mm does
*)
(*
showFactors
*)
(*
12 can be expressed uniquely two ways: a sum of terms 10+2 which is \
"understood", and also a product of primes 1*2*2*3
*)
showFactors[ai_] := Module[{a, b, x, y, i, c, p, r, v, af},
    v = {{1, 1}}; af = ai; r = 1; b = 0; i = 1; c = a + 1;
    If[r == a, Return[v]]; v = FactorInteger@ai;
    If[ai >= 1, PrependTo[v, {1, 1}]];
    Print@
      DisplayForm@
        RowBox@{Sequence @@ (v /. {x_Integer, y_Integer} -> 
                    SuperscriptBox[x, y])};
    v
    ]
(*
changeBaseArr
*)
changeBaseArr[np_, bp_] := Module[{n = np, b = bp, top = 1, arr = {}},
    While[n >= b, AppendTo[arr, Mod[n, b]]; n = n/b; n = IntegerPart[n]; 
      If[++top > 100000, Return[{}]]];
    AppendTo[arr, n];
    Reverse[arr]
    ]
(*
caesar  p123
*)
(*
note use of 255 not 26 since CharacterCode is ascii
*)
caesar[str_, enc_:True] := Block[{s = str, v},
    f[p_] := Mod[(p + 3), 255];
    fd[p_] := Mod[(p - 3), 255];
    If[enc,
      (* Print["s=", s]; *)
      v = ToCharacterCode@s;
      s = FromCharacterCode@f[v];
      (* Print["crypt=", s]; *)
      s
      ,
      v = ToCharacterCode@s;
      s = FromCharacterCode@fd[v];
      (* Print["s=", s] *)
      s
      ]
    ]
(*
congQ  pg 120
*)
k =.;
SetAttributes[congQ, Listable]
SetAttributes[congQM, Listable]
(*
book says for m|(a-b) m Z+ , but the following denies it, shows sign follows \
m across a,b
*)
(*
{Mod[2,4],Mod[-2,4],Mod[-6,4],Mod[6,4],Mod[2,-4],Mod[-2,-4],Mod[-6,-4],Mod[6,-\
4]}
*)
(*
{2,2,2,2,-2,-2,-2,-2}
*)
(*
also, the case m1 != m2  doesn't matter since it's not related to cong either \
way
technically try Mod1 since we do not know a,b,m came from Floor not \
IntegerPart
(it's commented out to make things "easier")
*)
congQM[a_, b_, m_] := Mod[a, m] == Mod[b, m]
congQM1[a_, b_, m_] := Mod1[a, m] == Mod1[b, m]
congQMboth[a_, b_, m_] := Mod[a, m] == Mod[b, m] || Mod1[a, m] == Mod1[b, m]
(*
m|(a-b) or a==b+km same test
*)
congQ[a_, b_, m_] := 
  Block[{r}, r = ((a - b)/m); IntegerQ@r] /; {a, b, m} \[Element] Integers && 
      m != 0
(*
NameQ show only k is left, thus m | (a-b) result was integer
	or, if dealing with fractions and r is integer, m divided
	unfortunately if N[] is involved there's maybe no way of checking if m \
divides (a-b)
*)
congQ[a_, b_, m_] := 
  Block[{r}, r = ((a - b)/m); NameQ@(ToString@r) || r == 0 || IntegerQ@r] /; 
    m != 0
(*
congQ[a_,b_,m_]:=(Print@"forced";congQ[Rationalize@a,Rationalize@b,\
Rationalize@m])/;m!=0&&(m\[NotElement]Integers)
*)
k =.
(*
congX , congA ,  3 x \[Congruent] Mod[4,7] , find x  p140
*)
(*
tested in section: function tests
*)
k =.;
(*
note.  as per mm book's explination: 0 x in eq. are replaced by 0 (ie, 0 \
Mod[4,0]==0) (evaluation reduces expr as soon as possible and 0's are special \
for time sake).  if these functions return unevaluated * 0 this 0 will be \
sometimes WRONG if you them inside a cong expression (if return not checked)
*)
(*
	m!=0 as congQ won't test it and if there is not div there can be no \
remainder to compare
*)
(*
\!\(congX[b_, a_, m_, k_:  Symbol["Global`k"]] := 
    If[a == 0, 0, \((\((b + k\ m)\)\/a)\)] /; {a, b, m} \[Element] Integers && 
        m > 1 && GCD[b, m] == 1 && \[Not] \((b == 0 && GCM[a, m] == 1)\)\)
*)
\!\(congX[b_, a_, m_, 
      k_Integer] := \(If[a == 0, 0, \((\((b + k\ m)\)\/a)\)] /; 
        a != 0 && m != 
            0 && \[Not] \((b == 0 && 
                GCD[Rationalize@a, Rationalize@m] == 1)\)\) /; 
      k \[Element] Integers\)
\!\(congX[b_, a_, m_, k_Symbol] := 
    If[a == 0, 0, \((\((b + k\ m)\)\/a)\)] /; 
      a != 0 && m != 
          0 && \[Not] \((b == 0 && 
              GCD[Rationalize@a, Rationalize@m] == 1)\)\)
\!\(congX[b_, a_, 
      m_] := \((Symbol["\<cr`k\>"]; 
        If[a == 0, 0, \((\((b + cr`k\ m)\)\/a)\)])\) /; 
      a != 0 && m != 
          0 && \[Not] \((b == 0 && 
              GCD[Rationalize@a, Rationalize@m] == 1)\)\)
(*
congA[b_,m_,k_:Symbol["Global`k"]]:=((b+k \
m))/;{a,b,m}\[Element]Integers&&m>1&&GCD[b,m]==1
*)
congA[b_, m_, k_Integer] := ((b + k m)) /; k \[Element] Integers && m != 0
congA[b_, m_, k_Symbol] := ((b + k m)) && m != 0
congA[b_, m_] := (Symbol["cr`k"]; (b + cr`k m)) && m != 0
k =.;
(*
crbruteforce , crtestnotcong , crtestcong
*)
(* find n cong brute force search *)

crbruteforce[bi_, mi_, n_:3, x_:0, lim_:5000] :=
  
  Module[{i, j, v, hit, c}, v = {}; c = 0;
    Do[hit = False;
      Do[If[True =!= congQ[j + x, bi[[i]], mi[[i]]],
            hit = True];
        , {i, Length@mi}];
      If[\[Not] hit, ++c; v = Append[v, j + x]];
      If[c >= n, Return[v]];
      , {j, 0, lim}
      ]; v]
(* show any ai cong to bi mod mi *)

crtestcong[bi_, mi_, ai_, dbg_:False] :=
  Module[{i, j, a, v, hit}, v = {};
    If[Head[ai] =!= List, a = {ai}, a = ai];
    Do[hit = False;
      Do[If[congQ[a[[j]], bi[[i]], mi[[i]]],
            If[dbg, v = Append[v, {a[[j]], bi[[i]], mi[[i]]}]];
            , hit = True;];
        , {i, Length@mi}];
      If[\[Not] dbg && \[Not] hit, v = Append[v, a[[j]]]];
      , {j, Length@a}];
    v]
(* show any ai not cong to bi mod mi *)

crtestnotcong[bi_, mi_, ai_, dbg_:False] :=
  
  Module[{i, j, a, v, hit}, v = {};
    If[Head[ai] =!= List, a = {ai}, a = ai];
    Do[hit = False;
      Do[If[True =!= congQ[a[[j]], bi[[i]], mi[[i]]],
            If[dbg, v = Append[v, {a[[j]], bi[[i]], mi[[i]]}]];
            hit = True;];
        , {i, Length@mi}];
      If[\[Not] dbg && hit, v = Append[v, a[[j]]]];
      , {j, Length@a}];
    v]
(*
inverseMod  by euclextgcd
*)
SetAttributes[inverseMod, {Listable, NumericFunction}]
inverseMod[b_, -1, m_] := inverseMod[b, m] /; Abs@m >= 1 && GCD[b, m] == 1
inverseMod[pm2_Integer, pm1_Integer] :=
  Module[{i, j, m2, m1, sm2, sm1, v},
      (* exclude 0's and 1's sol *)
      v = {}; If[pm1 == 0, Return[]];
      sm1 = Sign@pm1; sm2 = Sign@pm2;
      If[pm1 == sm1, If[pm2 === 1, v = 1, v = 0]];
      If[pm2 == 0, If[pm1 != sm1, Return[]]; v = 0];
      If[pm2 == sm2 && v == {}, 
        If[sm2 === -1 && sm1 === 1, v = pm1 - 1, v = sm2]];
      If[v =!= {}, Return@v];
      (* want m2 a - m1 b == 1 not sa + tb == 1 *)
      
      v = euclextgcd[pm2, -pm1];
      If[ListQ@v, {i, j} = v[[2]];
        (* see note *)
        If[Abs@(pm1 - i) + Abs@(pm2 - j) >
            Abs@(pm1 + i) + Abs@(pm2 + j), i = -i; j = -j];
        If[i pm2 - j pm1 != 1, i = pm1 - i; j = pm2 - j];
        Return@i];
      Print@"exhausted";
      ] /; Abs@pm1 >= 1 && GCD[pm1, pm2] == 1
(*
note: soln is not unique.  euclextgcd returns a lowest answer for sa-tb==1.  \
however the other side (b==1->b==-1 using {m1-i,m2-j}) isn't insured so.  \
after correction may be b==1 side.  PowerMod insures the other side is the \
lesser of two choices.
*)
(*
creqs  define
*)
crchart3[bi_, mi_, xi_:0, xf_:32, show_:True] :=
  
  Module[{i, len, gs}, len = Length@mi;
    Do[g[i] = 
        Plot[Mod[b, mi[[i]]] - Mod[bi[[i]], mi[[i]]], {b, xi, xf}, 
          PlotPoints -> 1000, DisplayFunction -> Identity, 
          PlotStyle -> {RGBColor[Random[], Random[], Random[]]}]
      , {i, len}];
    gs = Show[Table[g[i], {i, len}], DisplayFunction -> Identity];
    gs[[2]] = 
      gs[[2]] /. 
        Rule[DisplayFunction, Identity] -> 
          Rule[DisplayFunction, $DisplayFunction];
    If[show, Show[gs], gs]
    ]
combpairs[pairs_, stack_] := (
    If[pairs == {}, Return[{stack}]];
    combpairs[Rest@pairs, Append[stack, (First@pairs)[[1]]]]~Join~
      combpairs[Rest@pairs, Append[stack, (First@pairs)[[2]]]]
    )
creqs[pbi_, pmi_, eq_:True] :=
  
  Module[{i, len, bi, mi, r, h, leq, req, sr, eqs, sa},
    len = Length@pmi;
    If[Times @@ pmi == 0, Return[{}]];
    If[len != Length@pbi, Message[creqs::"lengths"]; Return[{}]];
    bi = pbi; mi = pmi;
    If[len == 1, bi = Join[bi, bi]; mi = Join[mi, mi]; len = 2];
    Do[c[i] = 0; c[i] =., {i, len}];
    leq = 
      Reverse@Sort[Table[{mi[[i]], bi[[i]]}, {i, len}], 
          OrderedQ[Abs@{#1, #2}] &];
    mi = leq[[All, 1]];
    bi = leq[[All, 2]];
    r = Mod[bi, mi];
    h = (mi - r);
    (* Table[If[bi[[i]] == 0, 0, 1], {i, len}]; *)
    c =.;
    If[eq === 1 || eq === True,
      req = mi Table[c[i], {i, len}]; 
      leq = req - h;
      sr = Transpose@{leq, req};
      If[eq === 1,
        eqs = sr[[All, 1]];
        eqs = {Equal @@ eqs};
        ];
      If[eq === True,
          eqs = combpairs[sr, {}];
          (* sa = {}; *)
          eqs = Map[(Equal @@ #) &, eqs];
          ];
      ];
    {{bi, mi, r, h}, {leq, req, sr, eqs}}
    ]
(*
legend:
	(2) mi is tot. width of one saw tooth (above and below x axis) (and first R)
	(3) r==tw-wx is first x axis position of L side of tooth (== mod remainder)
	(4) mi-r is width==height of tooth above x axis
	(* minimal build only the L,L,L soln eq *)
	(* sa=leq/mi;sa={Equal@@sa};eqs=leq;eqs={Equal@@eqs}; *)
	old format was: {bi,mi,h,leq,req,{},r,{,},{},sr,eqs,sa}
*)
(* function shows how many triangles at x == 30 *)

crxtotriag[mi_, x_] := (1/mi) x
(* function shows x given num triag *)
crtriagtox[mi_, x_] := mi x
(*
note crxtotriag is special use it is Not a kind of accurate unit conversion.  \
it's close for small nums.
*)
(*
crfindnSolnAfterxInEqm  define
*)
crfindnSolnAfterxInEqm[pbi_, pmi_, n_:3, xi_:0, slim_:False, mp_:1, 
    modchk_:False, dbg_:False] :=
  
  Module[{i, j, len, mi, bi, r, h, x, y, z, ans, eqn, sol, arr, xf, t2, s0, 
      t3, t1, numeqs, mst, mend, xst, xend, dir, m, ints, ok, step, off, 
      trlim, final, cc, ad},
    i = 0; sol = 0;
    If[mp === 1,
      t1 = creqs[pbi, pmi, 1],
      t1 = creqs[pbi, pmi]
      ];
    If[Length@t1 <= 1, Return[t1]]; If[Length@t1[[1]] <= 1, Return[t1]];
    {bi, mi, r, h} = t1[[1]]; t1 = t1[[2]];
    numeqs = Length@t1[[4]];
    If[numeqs < 2 ^len, Return[{}]];
    len = Length@mi;
    (*ad = LCM @@ mi;*)
    
    If[False && mp == 1, 
      If[crexclude[bi, mi, r, h, len] == False, Return[{}]]];
    If[mp === All,
      mst = 1; mend = numeqs,
      mst = mp; mend = mp;
      If[numeqs < mp, Message[crfindnSolnAfterxInEqm::"noeqn", mp]; 
        Return[{}]]
      ];
    If[slim === False,
      trlim = Abs@(n IntegerPart@N@crxtotriag[mi, Times @@ mi][[1]]) + 1,
      trlim = slim
      ];
    Do[c[i] = 0; c[i] =., {i, len}];
    cc = Table[c[i], {i, len}];
    (* change x units to triag units *)
    
    t2 = t1[[4, mst]];(*creqs[][[2, 4, mst]]*)
    
    x = c[1] /. Solve[t2[[1]] == xi, c[1]][[1]];
    xf = Floor@x;
    t3 = c[1] /. Solve[t2[[1]] == xi + 1, c[1]][[1]];
    (* auto flip if it will run other way *)
    If[x > t3,
      xst = xf; xend = xf - trlim; dir = -1;
      , xst = xf; xend = xf + trlim; dir = 1;
      ];
    t3 =.; x =.; If[xf == 0 && xi != 0, ++xf];
    If[modchk,
      ints = False, ints = IntegerQ /@ mi === Table[True, {i, len}];
      If[ints, ints = IntegerQ /@ bi === Table[True, {i, len}]];
      ];
    final = {}; arr = {};
    Do[
      (* ok to use t11m. 
            convert from triag to x using t2 then check soln was on axis \
creqs[][[2, 4, m]] *)
      arr = {}; sol = 0;
      t2 = t1[[4, m]];
      t3 = Solve[t2, Rest@cc];
      eqn = Table[c[i], {i, 2, len}] /. t3;
      Do[
        i = id;
        ans = eqn /. c[1] -> i;
        If[Length@ans >= 1, ans = ans[[1]], Continue[]];
        z = (t2[[1]] /. c[1] -> i);
        If[ints,
          If[IntegerQ@z,
            ans = Join[ans, {z}];
            If[IntegerQ /@ ans === Table[True, {j, len}],
              AppendTo[arr, z]; ++sol
              ]]
          ,
          
          If[congQ[z, r, mi] === Table[True, {j, len}], 
              AppendTo[arr, z]; ++sol];
          ];
        If[sol >= n, Break[]];
        , {id, xst, xend, dir}
        ];
      If[arr != {}, AppendTo[final, arr]];
      , {m, mst, mend}];
    If[sol < n, Message[crfindnSolnAfterxInEqm::"exhausted"]];
    If[Length@final == 1, arr, final]
    ]
(*
quirk: neg bi,mi can run opposite direction of subst so this "fixes".  it's \
very ornary to fix if it doesn't work remove all fixes do after exit.  note \
trlim is loop limit not search boundry.
*)
(*
cralign1
*)
(*
see align1
*)
(*
cralign1 doesn't know if we align all Rs or all Ls of sawtooth triangles (how \
data is supplied). crtwo uses all Ls aligned.
*)
cralign1[m1_, o1_, m2_, bothoff_:0] := Module[{i, j, gcd, x},
    If[Mod[o1, m2] == 0, Return[o1 + bothoff]];
    gcd = GCD[m1, m2];
    If[gcd === 1,
      j = PowerMod[-m1, -1, m2];
      If[j != 0, j = Mod[o1 j, m2]];
      Return[j m1 + o1 + bothoff]
      ,
      If[Mod[o1, gcd] == 0,
          x = m2/gcd;
          j = PowerMod[-m1/gcd, -1, x];
          If[j != 0, j = Mod[(o1/gcd) j, m2/gcd]];
          Return[j m1 + o1 + bothoff]
          ];
      ];
    ]
(*
more true: If[m2<=o1&&Mod[o1,m2]==0,Return[o1+bothoff]]; ... \
If[Mod[o1,m2]===0,j=o1-m2.  this is inner loop of cr: for good timings do the \
least work possible, "truer values" do not change cr, so just return 0.
*)
(*
if o1>m1 do not answer (a wave isn't longer than itself)(or bad, \
o1=Mod[o1,m1]).  however i don't check because: 1) speed critical  2) if \
crtwo isn't broken it won't ask  3) crtwo uses negative offsetting (actually \
just barfs data) and this function doesn't know that and if it did it \
wouldn't be it's job.  4) can't have it both ways it can't both quit but \
still work with negatives, ie...
	(* If[Abs@o1>Abs@m1,Return[]]; *)
*)
(*
align1
*)
(*
see "plots, align1 tab, ..."
*)
align1[m1_, o1_, m2_, bothoff_:0] := Module[{i, j, gcd, x},
    If[Mod[o1, m2] == 0, Return[{o1/m2, 0, o1 + bothoff}]];
    gcd = GCD[m1, m2];
    If[gcd === 1,
      j = PowerMod[-m1, -1, m2];
      If[j != 0, j = Mod[o1 j, m2]];
      i = (j m1 + o1);
      Return[{i/m2, j, i + bothoff}]
      ,
      If[Mod[o1, gcd] == 0,
          x = m2/gcd;
          j = PowerMod[-m1/gcd, -1, x];
          If[j != 0, j = Mod[(o1/gcd) j, m2/gcd]];
          i = (j m1 + o1);
          Return[{i/m2, j, i + bothoff}]
          ];
      ];
    ]
(*
cralign1 align1   table
*)
testa1[m1_, m2_, pstart_:True, pend_:True] := Module[{start, end, i, j, v},
    start = pstart; end = pend;
    If[pstart, start = -Abs@(m1) - 2];
    If[pend, end = Abs@(2 m1) + 2];
    Table[v = align1[m1, i, m2]; {m2 , v[[1]], v[[2]], m1, i, v[[3]]}
      , {i, start, end}]
    ]
testcra1[m1_, m2_, pstart_:True, pend_:True] := Module[{start, end, i, j, v},
    start = pstart; end = pend;
    If[pstart, start = -Abs@(m1) - 2];
    If[pend, end = Abs@(2 m1) + 2];
    Table[{m2 , m1, i, cralign1[m1, i, m2]}
      , {i, start, end}]
    ]
(*
crtwo
*)
(*
	a little longer, incl. for redundancy
*)
crtwo2[bi_, mi_, dbg_:False] := 
  Module[{i, j, len, r, h, a4, ad, b1, b2, m1, m2, r1, r2},
    len = Length@mi;
    If[len != Length@bi, Message[crtwo::"lengths"]; Return[{}]];
    m1 = mi[[1]]; If[m1 == 0, Return[{Null, Null}]];
    b1 = bi[[1]]; r1 = Abs@Mod[b1, Abs@m1]; i = 0;
    If[len == 1, Return@crtwo[{b1, b1}, {m1, m1}, dbg]];
    Do[
      m2 = mi[[j]]; If[m2 == 0, b1 = Null; m1 = Null; Break[]];
      r2 = Abs@Mod[bi[[j]], Abs@m2];
      If[dbg, ++i;
        Print@{"obj ", 
            LCM[m1, m2], {Abs@m1, r1 - r2, Abs@m2, r2}, {b1, bi[[j]]}, {m1, 
              m2}, {r1, r2}, i};
        Print@{"cr2 cr",
            If[r1 > r2, b1 = cralign1[Abs@m1, r1 - r2, Abs@m2, r2],
              b1 = cralign1[Abs@m2, r2 - r1, Abs@m1, r1]]
            , crtlist[{b1, bi[[j]]}, {m1, m2}, 1, 2]
            }
        ];
      If[r1 > r2,
        b1 = cralign1[Abs@m1, r1 - r2, Abs@m2, r2],
        b1 = cralign1[Abs@m2, r2 - r1, Abs@m1, r1]
        ];
      If[b1 == Null, m1 = Null; Break[]];
      m1 = LCM[m1, m2]; r1 = Mod[b1, m1],
      {j, 2, len}];
    {b1, m1}]
(*
the trick: the last answer becomes what the next pair must align with (if \
congruence is possible).  (choice of b1+ rather than least abs, is arbitrary)
*)
crtwo[bi_, mi_, dbg_:False] :=
  Module[{i, j, len, b1, b2, m1, m2, r1, r2},
    len = Length@mi;
    If[len != Length@bi, Message[crtwo::"lengths"]; Return[{}]];
    m1 = mi[[1]]; If[m1 == 0, Return[{Null, Null}]];
    b1 = bi[[1]]; r1 = Mod[b1, m1]; i = 0;
    If[len == 1, Print@{b1, m1}; Return@crtwo[{b1, b1}, {m1, m1}, dbg]];
    Do[
      m2 = mi[[j]]; If[m2 == 0, b1 = Null; m1 = Null; Break[]];
      r2 = Mod[bi[[j]], m2];
      If[dbg, ++i;
        Print@{"obj ", 
            LCM[m1, m2], {m1, r1 - r2, m2, r2}, {b1, bi[[j]]}, {m1, m2}, {r1, 
              r2}, i};
        Print@{"cr2 cr", cralign1[m1, r1 - r2, m2, r2], 
            crtlist[{b1, bi[[j]]}, {m1, m2}, 1, 2]}
        ];
      b1 = cralign1[m1, r1 - r2, m2, r2];
      If[b1 == Null, Break[]];
      m1 = LCM[m1, m2]; r1 = Mod[b1, m1],
      {j, 2, len}];
    If[b1 == Null, Return[{Null, Null}]];
    If[b1 < 0, {b1 + m1, m1}, {b1, m1}]
    ]
(*
crAll
*)
(*
theory:
	soln repeat every n beginning at m , no matter how many vectors are involved
*)
crAll[bi_, mi_, n_:5, x_:0, lim_:False, m_:1] := 
  Module[{r, i, xi, avediff, sign, v},
    v = Flatten@crfindnSolnAfterxInEqm[bi, mi, 2, x, lim, m];
    If[Length@v < 2, Return[v]]; avediff = LCM @@ mi;
    r = First@v; xi = Ceiling[(x - r)/avediff];
    Table[r + (i avediff), {i, xi, xi + n - 1}]
    ]
crAll2[v_, n_:5, x_:0, lim_:False] := Module[{r, i, xi, avediff, sign},
    r = First@v; If[r == Null, Return[{}]]; avediff = Last@v;
    xi = Ceiling[(x - r)/avediff];
    Table[r + (i avediff), {i, xi, xi + n - 1}]
    ]
(*
IntToModTups  AddModTups  TupsModToInt  p142
*)
IntToModTups[n_, v_:{99, 98, 97, 95}] := Module[{vp},
    vp = Times @@ v; If[\[Not] IntegerQ@vp, Return[]];
    If[n > vp, Return["n too large for v"]];
    Map[(Mod[n, #]) &, v]
    ]
AddModTups[n_, m_, v_:{99, 98, 97, 95}] := 
  Module[{vp, fv}, vp = n + m; vp~Mod~v]
SubModTups[n_, m_, v_:{99, 98, 97, 95}] := 
  Module[{vp, fv}, vp = n - m; vp~Mod~v]
MulModTups[n_, m_, v_:{99, 98, 97, 95}] := 
  Module[{vp, fv}, vp = n*m; vp~Mod~v]
(*
	use mag as sign, saves effort in tracking separately
*)
TupsModToInt[n_, v_:{99, 98, 97, 95}, signed_:True] := 
  Module[{x, z}, x = ChineseRemainder[n, v]; If[\[Not] IntegerQ@x, Return[]]; 
    z = Times @@ v; If[signed && x >= z/2, x - z, x]]
(*
end package
*)
End[];
On[General::"spell"] (* turn off has similar spelling messages *)
EndPackage[];
