(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     57731,       1469]*)
(*NotebookOutlinePosition[     65824,       1708]*)
(*  CellTagsIndexPosition[     64177,       1660]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[":Mathematica Version: 4.0", "Text"],

Cell[":Package Version: 1.1", "Text"],

Cell[":Copyright: public domain", "Text"],

Cell[":Name: cr`", "Text"],

Cell[":Title: n chinese remainders", "Text"],

Cell[":Author: John Hendrickson (debguy@sourceforge.net)", "Text"],

Cell[":History: .1 minor packaging improvement", "Text"],

Cell["\<\
:Keywords:
        number theory, Chinese Remainder Theorem,
        euclalg, euclalglist, euclbacksolve, euclextgcd, inverseMod,
        testgen, showFactors, changeBaseAArr, caesar
        crtwo, cralign1, align1,
        crchart3, creqs, crfindnSolnAfterxInEqm
        crAll, crAll2, IntToModTups,
        congQ, congX, crbruteforce, crtestnotcong, crtestcong\
\>", "Text"],

Cell[":Requirements: None", "Text"],

Cell[":Sources: all code is origional by author", "Text"],

Cell["\<\
:Summary:

\tPresenting:

\tcrtwo: solves Chinese Remainder by simple pair at a time
\t\tcrAll2@crtwo provides n soln near x for CR, uses cr width
\t\tcrchart3 shows soln as sawtooth waves (CR) meeting
\talign1: solves i m2 == j m2 + b -> {i,j,dist}
\t\tlike PowerMod->i but b is any and gcd(m1,m2) not req.
\tother funs:
\t\tEuclidians, showFactors, changeBaseArr, caesar, rsa, 
\t\tinverseMod, IntToModTups (large number math for PC's)
\t\tcrfindnSolnAfterxInEqm (solve saw wave LL,RL,LR,... intersects)
\t\tcongruence, plots, y-shifted triangular wave solve, testing

\tby: John Hendrickson (debguy@sourceforge.net), 2013, public domain
\t\tcredit to many authors of math in general

\tpartial book notes were from:
\tRosen, Kenneth H., \"Discrete Mathematics and it's Applications\", 3rd ed.

see divs.nb (n-chinese-remainders.nb) for use, plots and discussions, and a \
few extra funs\
\>", "Text"],

Cell[CellGroupData[{

Cell["setup", "Subsubsection"],

Cell[BoxData[
    \(\(\( (*\ suggested\ , \ 
      see\ the\ mm\ book\ *) \)\(\[IndentingNewLine]\)\(If[
      False, \[IndentingNewLine]Off[General::"\<spell\>"];  (*\ 
        turn\ off\ has\ similar\ spelling\ messages\ *) \[IndentingNewLine] \
(*\ On[MemoryConserve]\ *)  (*\ 
        may\ be\ causing\ crashes\ *) \[IndentingNewLine]Unprotect[In, 
        Out]; \[IndentingNewLine]Clear[In, Out]; \[IndentingNewLine]Protect[
        In, Out]; \[IndentingNewLine]$HistoryLength = 
        5; \[IndentingNewLine]MemoryInUse[];\[IndentingNewLine]]\)\)\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Needs["\<NumberTheory`NumberTheoryFunctions`\>"];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(BeginPackage["\<cr`\>", {"\<NumberTheory`NumberTheoryFunctions`\>"}];\
\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(Off[General::"\<spell\>"]\)\(\ \)\( (*\ 
      turn\ off\ has\ similar\ spelling\ messages\ *) \)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Clear[\[IndentingNewLine]k, rand, testgen, \[IndentingNewLine]Mod1, 
      Mod2, Mod3, Mod4, Mod8, Mod5, \neuclalg, euclalg2, euclalglist, 
      euclbacksolve, euclextgcd, euclalglistUnsigned, euclextgcd, \n
      showFactors, \nchangeBaseArr, \ncaesar, \ncongQ, congQM, congQM1, 
      congQMboth, congX, congA, \ncrbruteforce, crtestcong, crtestnotcong, \n
      inverseMod, inverseMod2, \ncrchart3, \ncombpairs, creqs, crxtotriag, 
      crtriagtox, \ncrfindnSolnAfterxInEqm, \ncralign1, testcra1, align1, 
      testa1, \ncrtwo, crtwo2, chineseremainder, crtlist, \ncrAll, crAll2, \n
      IntToModTups, AddModTups, SubModTups, MulModTups, 
      TupsModToInt\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(cr::"\<usage\>" = "\<cr` is context for n-chinese-remainders (crtwo) \
keeps symbols.  see\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Mod1::usage = "\<Mod1-8 are for study purpose\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(euclalg::usage = "\<euclalg return should be same as GCD\>";\)\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(euclalglist::usage = "\<euclalglist returns {r,b,q} meaning \
{{a1,b1,0},{r1,b1,q1},{r2,r1,q2},...} for a=q*b+r each successive euclalg\>";\
\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(euclextgcd::usage = "\<euclextgcd return should be same as \
ExtendedGCD\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(showFactors::usage = "\<showFactors[n]  a product of primes display \
of n\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(changeBaseArr::usage = "\<changeBaseArr[n,b]  shows list of digits of \
number n in base b, the list is given in base 10\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(caesar::usage = "\<caesar[msg,encode] cipher and de-cipher (enc \
False). for study purpose\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(congQ::usage = "\<congQ[a,b,m] , m|(a-b) , congruentQ , a \
\[Congruent] Mod[b,m] , b \[Congruent] Mod[a,m] , a == b + k m , a or b can \
contain k (ie 4+7k) if m | (a - b) or only k is left or result is 0 (a==b).  \
if congQ returns \"expr==0\" for (ie a==4+7k) expr is not always cong but \
particular choices of k may cause r==0 or m|(a-b), which is neither T or F.  \
In general use congQ==True since \[Not]congQ is T when congQ did nothing.  \
for N@a consider using Rationalize@a.  see congQM\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[343]:="],

Cell[BoxData[
    \(\(congQM::usage = "\<congQM[a,b,m] checks congruence by \
Mod[a,m]==Mod[b,m].  note for neg num Mod assumes a,b,m work by Floor not \
IntegerPart (Mathematica a/m=q+r/m funs use Floor, which is ok if a,b,m are \
found using mm funs), see Mod details, congQM1, congQMboth\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[344]:="],

Cell[BoxData[
    \(\(congX::usage = "\<congX[b,a,m,k:k] (b-a k)=m c , return eq or value k \
, Mod[a k,m] \[Congruent] Mod[b,m] , congQ[a congX[b,a,m,k?],b,m] , for any \
choice k.  default for k is symbol k (subst after).  see also congA.  a right \
choice of k is inv if there is one, see inverseMod.\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[357]:="],

Cell[BoxData[
    \(\(congA::usage = "\<congA[b,m,k:k]  .5 broken.  return eq or value a k \
, a \[Congruent] Mod[b k, m] , congQ[congA[b,m],b,m] , for any choice k.  \
default for k is symbol k (subst after).  see also congX. use congX if b==0 \
then GCD[a,m]==1 needs to be tested. does not work with fractions.  not \
suggested\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[358]:="],

Cell[BoxData[
    RowBox[{
      RowBox[{\(inverseMod::usage\), "=", 
        "\"\<inverseMod[b,[-1],m]  \!\(a\^\(-1\)\) \[Congruent] Mod[x,m] ; a \
x == a \!\(TraditionalForm\`a\^\(-1\)\)\[Congruent] Mod[1,m] /; m!=0 && .  \
see also congX\>\""}], ";"}]], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[389]:="],

Cell[BoxData[
    \(\(crchart3::usage = "\<crchart3[bi,mi,xi:0,xf:30,show:True]  shows \
sawtooth waves that represent the chinese remainder theorem moduli (each wave \
is one pair). moduli have soln where they cross x axis and where all lefts on \
axis meet a soln lies.\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2719]:="],

Cell[BoxData[
    \(\(creqs::usage = "\<creqs[bi_:{2,5,6},mi_:{3,7,10},eq]  gives eqs for \
solving chinese remainder theorem that show where LL triangles meet axis, or \
any L or R pairs to meet (see crchart3). for Length 3 there are 2 2 2 eqs (m \
can be All though only m1's are cr soln).  eq==False is none, and 1 means \
only 1st, True is all.  {{bi,mi,r,h},{leq,req,sr,eqs}}.  see crchart3, \
crfindnSolnAfterxInEqm\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[405]:="],

Cell[BoxData[
    \(\(creqs::lengths = "\<bi mi are different lengths\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[406]:="],

Cell[BoxData[
    \(\(crfindnSolnAfterxInEqm::usage = \
"\<crfindnsolnaftertriagineqm[bi,mi,n:3,x:0,lim,m:1,modchk,dbg]  solves \
chinese remainder theorem by brute force per say.  however cr finds only Left \
L (see crchart3) of triangles, this fun can find L or R for each triag to \
each other L or R and can work with fractions.  caveats: likely slow if mi > \
10 items.  \"find the next n soln of eq m of given x offset\". lim is how \
many loops to try.  modchk True uses Mod, takes longer, but works with \
fractions (forced if not Integer).  m can be All (1 is cr, see creqs, if m!=1 \
there are 2^lenmi small expr, allot). see crchart3, crAll, crtwo.\>";\)\)], \
"Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2193]:="],

Cell[BoxData[
    \(\(crfindnSolnAfterxInEqm::"\<noeqn\>" = "\<creqs eq has only `1` eq\>";\
\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2194]:="],

Cell[BoxData[
    \(\(crfindnSolnAfterxInEqm::"\<nosolve\>" = "\<warn: creqs eq `1` does \
not solve\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2195]:="],

Cell[BoxData[
    \(\(crfindnSolnAfterxInEqm::"\<exhausted\>" = "\<exhausted searching , \
none found so far\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2196]:="],

Cell[BoxData[
    \(\(align1::usage = "\<align1[sawm1,offset,sawm2,bothoff,dbg]  solves m2 \
i == m1 j + offset, and returns {i,j,dist} and all are Integers, a useful \
extention to PowerMod(m2 i == m1 j + 1, returns i).  thinking of LCM[mi] as \
tot width to sawtooth waves meeting from one intersection to next and dist as \
meeting when offset is involved.  soln can be same as chinese remainder. \
(meaning sawtooth with x and y offsets that meet x axis)(PM funs are \
essential when working with math where solutions must also be Integer)  see \
also: crchart3, cralign1, align1non0\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[437]:="],

Cell[BoxData[
    \(\(cralign1::usage = "\<cralign1[sawm1,offset,sawm2,bothoff,dbg]  solves \
m2 i == m1 j + offset, and returns dist. thinking of LCM[mi] tot width to \
sawtooth waves: what about when the starting point is not aligned?  soln can \
be same as chinese remainder, dist.  for cralign j can be 0 and i,j,dist are \
Integers. (sawtooths having y offset known by x offset tooth widths. for \
plain sawtooths LCM@@mi is ans)  see also: crchart3, crtwo, crAll, \
crbruteforce align1\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[427]:="],

Cell[BoxData[
    \(\(crtwo::usage = "\<crtwo[bi,mi,dbg]  solves chinese remainder theorem, \
the sys of eq  x mod m \[Congruent] b mod m (lists b m, x scalar) and returns \
{x intersect, width to next}  benefits: x is always positive. gives width \
(see crAll). it quits much sooner when there is no soln (pair at a time)(dbg \
shows first non-congruent).  uses no memory except the input (CR creates \
lists).  comes with useful sawtooth & multiples aligners.  caveates: usu. \
takes twice as long or for large data (but is bigO linear), though beats CR \
on a few.  crtwo is faster on the average if more than 1/2 of solving \
attempts will not find a soln.  notes: CR and crtwo work only on Integer and \
give wrong answers for fractions (so multiply first) (all Rs of sawtooths \
align at 0: but CR can give negative ans).  see also: crchart3, cralign1, \
crAll2, crbruteforce\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[1013]:="],

Cell[BoxData[
    \(\(crtwo::"\<lengths\>" = "\<bi mi are different lengths\>";\)\)], \
"Input",
  InitializationCell->True,
  CellTags->"(lic2) In[1014]:="],

Cell[BoxData[
    \(\(crAll::usage = "\<crAll[bi,mi,n:5,x:0,m]  solves chinese remainder \
theorem, the sys of eq  x \[Congruent] b mod m (b and m are lists)  find the \
next n consecutive soln given x which is where to start looking (can be neg). \
 note soln may include x +- smallest value of modulus (smallest triangle \
size) that's chinese remainder soln.  crAll2 needs {intersect, width} instead \
of bi,mi.  see also: crchart3, crtwo, crbruteforce, crfindnSolnAfterxInEqm \
(finds first solns)\>";\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2186]:="],

Cell[BoxData[
    \(\(IntToModTups::usage = "\<IntToModTups[n,v]  n is converted by v, and \
after add sub mul, is converted back by TupsModToInt.  not well tested, for \
discussion purposes.  (to choose non-default v, m = m1 m2 m3 ... where mk > 2 \
and gcd(mi,mj)==1 when i!=j.  note TMTI uses CR not crtwo, whether nearest cr \
after x==0 or signed cr is right: is left in the air\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(\(k =. \) \)\(;\)\(\ \)\( (*\ congX\ needs\ to\ share\ k, \ 
      not\ in\ a\ context\ yet, \ 
      so\ Clear\ do\ not\ Remove\ *) \)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(rand[] := \((\((Random[]\ 20)\) - 10)\)\)], "Input"],

Cell["mm  chineseremainder  and  crtlist", "Text"],

Cell["\<\
mm 4.0 contains ChineseRemainder and CRTlist in.  crlist is \
normally tucked in private.  see  \
mathematica/AddOns/StandardPackages/NumberTheory/NumberTheoryFunctions.m\
\>",\
 "Text"],

Cell[BoxData[
    \(\(chineseremainder = ChineseRemainder;\)\)], "Input"],

Cell[BoxData[
    \(\(crtlist = 
        NumberTheory`NumberTheoryFunctions`Private`CRTlist;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"];\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["testgen", "Subsubsection"],

Cell["\tgeneric brute force test function", "Text"],

Cell[BoxData[
    \(rand[] := \((\((Random[]\ 20)\) - 10)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(testgen[f1_, f2_, n_:  100, dbg_:  False] := 
      Module[{a, b, m, last, next, rR, 
          rI}, \[IndentingNewLine]Do[\[IndentingNewLine]a = rand[]; 
          b = rand[]; 
          m = rand[]; \[IndentingNewLine]a = 
            Rationalize[a,  .01]; \[IndentingNewLine]b = 
            Rationalize[b,  .01]; \[IndentingNewLine]m = 
            Rationalize[m,  .01]; \[IndentingNewLine]If[
            f1[a, b, m] != f2[a, b, m], 
            Print@{a, b, m, f1[a, b, m], 
                f2[a, b, m]}]; \[IndentingNewLine]a = Floor@a; b = Floor@b; 
          m = Floor@m; \[IndentingNewLine]If[f1[a, b, m] != f2[a, b, m], 
            Print@{a, b, m, f1[a, b, m], 
                f2[a, b, 
                  m]}];\[IndentingNewLine], {n}]\[IndentingNewLine]]\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \(testgen[f1_, f2_, n_:  100, ruleR_, ruleI_, dbg_:  False] := 
      Module[{a, b, m, last, next, rR, 
          rI}, \[IndentingNewLine]Do[\[IndentingNewLine]a = rand[]; 
          b = rand[]; 
          m = rand[]; \[IndentingNewLine]a = 
            Rationalize[a,  .01]; \[IndentingNewLine]b = 
            Rationalize[b,  .01]; \[IndentingNewLine]m = 
            Rationalize[m,  .01]; \[IndentingNewLine]{as, bs, ms} = 
            Abs@{a, b, m}; \[IndentingNewLine]If[ruleR[a, b, m], 
            If[f1[a, b, m] != f2[a, b, m], 
              Print@{a, b, m, f1[a, b, m], 
                  f2[a, b, m]}]]; \[IndentingNewLine]a = Floor@a; 
          b = Floor@b; 
          m = Floor@m; \[IndentingNewLine]{as, bs, ms} = 
            Abs@{a, b, m}; \[IndentingNewLine]If[ruleI[a, b, m], 
            If[f1[a, b, m] != f2[a, b, m], 
              Print@{a, b, m, f1[a, b, m], 
                  f2[a, b, 
                    m]}]];\[IndentingNewLine], {n}]\[IndentingNewLine]]\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \(testgen[f1_, f2_, n_:  100, ruleR_, ruleI_, dbg_:  False] := 
      Module[{a, b, m, last, next, rR, 
          rI}, \[IndentingNewLine]Do[\[IndentingNewLine]a = rand[]; 
          b = rand[]; 
          m = rand[]; \[IndentingNewLine]a = 
            Rationalize[a,  .01]; \[IndentingNewLine]b = 
            Rationalize[b,  .01]; \[IndentingNewLine]m = 
            Rationalize[m,  .01]; \[IndentingNewLine]{as, bs, ms} = 
            Abs@{a, b, m}; \[IndentingNewLine]If[ruleR[a, b, m], 
            If[f1[a, b, m] != f2[a, b, m], 
              Print@{"\<hi\>", a, b, m, f1[a, b, m], 
                  f2[a, b, m]}]]; \[IndentingNewLine]a = Floor@a; 
          b = Floor@b; 
          m = Floor@m; \[IndentingNewLine]{as, bs, ms} = 
            Abs@{a, b, m}; \[IndentingNewLine]If[ruleI[a, b, m], 
            If[f1[a, b, m] != f2[a, b, m], 
              Print@{a, b, m, f1[a, b, m], 
                  f2[a, b, 
                    m]}]];\[IndentingNewLine], {n}]\[IndentingNewLine]]\)], \
"Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mod 1-5,8", "Subsubsection"],

Cell["\<\
\tMod1[b,m] shows that for neg nums there are two right answers.
\t\tthe difference is whether a/m=q+r/m whether
\t\tq and r use Floor or IntegerPart (q and r must use same)
\tThe book and mathematica choose positive remainders, Mod1 chooses other.
\t\tn Quotient[m, n, d] + Mod[m, n, d] == m\
\>", "Text"],

Cell[BoxData[
    \(Mod1[a_, m_] := a - m\ IntegerPart[a/m]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Mod2[a_, m_] := 
      m/2 - \((m/Pi)\)\ ArcTan[Cot[\((Pi\ a)\)/m]] /; 
        a/m \[Element] Reals && \(! a/m \[Element] Integers\)\)], "Input",
  InitializationCell->True],

Cell["\tnote Mod2 and Mod4 do not, in general, give all soln", "Text"],

Cell[BoxData[
    \(Mod2[a_, m_] := 
      0 /; m \[NotEqual] 0 && a/m \[Element] Integers\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Mod3[a_, m_] := a - m\ Floor[a\/m]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Mod4[a_, m_] := 
      Last@IntegerDigits[a, m] /; 
        a \[Element] Integers && m \[Element] Integers && m > 1 && 
          a > 1\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Mod4[a_, m_] := a - m\ Quotient[a, m]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Mod8[a_, m_] := a - m\ Quotient[a, m]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(SetAttributes[Mod5, {Listable, NumericFunction}];\)\)], "Input",
  InitializationCell->True],

Cell["\<\
purpose of Mod5 is to show Sign needs and result needs. very slow \
if b << a (by more than a few powers of 10)  \t(Mod5 doesn't approx, does do \
frac)\
\>", "Text"],

Cell[BoxData[
    \(Mod5[a_, m_] := 
      Module[{x, y, ms}, \[IndentingNewLine]If[m \[Equal] 0, Return[]]; 
        If[a \[Equal] 0, Return[0]]; \[IndentingNewLine]If[Sign@m \[Equal] 1, 
          If[Sign@a \[Equal] 1, \[IndentingNewLine]If[a < m, 
              Return[a]]; \[IndentingNewLine]y = \((a - m)\); 
            While[y >= m, y -= m]; Return[y]\[IndentingNewLine], 
            y = \((m + a)\); While[y < m, y += m]; 
            Return[y - m]]\[IndentingNewLine], 
          If[Sign@a \[Equal] 1, \[IndentingNewLine]y = \((m + a)\); 
            While[y > m, y += m]; Return[y - m]\[IndentingNewLine], 
            If[a > m, Return[a]]; \[IndentingNewLine]y = \((a - m)\); 
            While[y <= m, y -= m]; Return[y]\[IndentingNewLine]]]]\)], "Input",\

  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Euclidian Algorithms  gcd  gcdlist  extended gcd", "Subsubsection"],

Cell["simple gcd and extended gcd (pairs, no List)", "Text"],

Cell["\<\
divides until least remainder is found for a1,a2
\ta1 = q1 * b1 + r1
\tb1 = q2 * r1 + r2
\tr1 = q3 * r2 + r3 ...\
\>", "Text"],

Cell["\<\
euclalg[m2_,m1_]:=(If[m2\[Equal]0,Return[m1]];euclalg[m1-m2 \
Quotient[m1,m2],m2])\
\>", "Text"],

Cell[BoxData[
    \(euclalg[m2_, m1_] := 
      Module[{x, r, q}, q = Abs@m1; r = Abs@m2; 
        If[q < r, x = q; q = r; r = x]; If[r \[Equal] 0, Return@q]; 
        While[True, x = Mod[q, r]; If[x \[Equal] 0, Return[r]]; q = r; 
          r = x]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(euclalg2[{m2_, m1_, c_}] := \((If[m2 \[Equal] 0, 
          Return[{0, 0, 0}]]; {Mod[m1, m2], m2, 
          Quotient[m1, m2]})\)\)], "Input",
  InitializationCell->True],

Cell["\<\
\treturn is sequence of {r,b,q} ; a==m1, b==m2, a == q * b + \
r\
\>", "Text"],

Cell[BoxData[
    \(euclalglist[pm2_, pm1_] := 
      Module[{m2, m1, x}, \[IndentingNewLine]If[
          euclalg2[{0, 0, 0}] \[NotEqual] {0, 0, 0}, Print@"\<err\>"; 
          Return[]]; \[IndentingNewLine]m1 = Abs@pm1; m2 = Abs@pm2; 
        If[m1 < m2, x = m1; m1 = m2; m2 = x]; 
        If[m2 \[Equal] 0, Return@m1]; \[IndentingNewLine]NestWhileList[
          euclalg2, {m2, m1, 
            0}, \((First@#1 != 0)\) &]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(euclbacksolve[last_, this_, n_, v_] := 
      Module[{}, \[IndentingNewLine]If[n >= Length@v, 
          Return[{last, this}]]; \[IndentingNewLine]euclbacksolve[this, 
          last - this\ v[\([n + 1]\)], n + 1, 
          v]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
v=Table[b[i],{i,1,5}];
euclbacksolve[1,-v[[3]],3,v]
should do this:
     {1+b[3] b[4],-b[3]-(1+b[3] b[4]) b[5]}\
\>", "Text"],

Cell[BoxData[
    \(euclextgcd[pm2_Integer, pm1_Integer] := 
      Module[{i, j, x, m2, m1, sm2, sm1, signdiff, m2more, v, 
          gcd}, \[IndentingNewLine]If[pm1 \[Equal] 0, 
          Return[{Abs@pm2, {Sign@pm2, 0}}]]; \[IndentingNewLine]If[
          pm2 \[Equal] 0, 
          Return[{Abs@pm1, {0, Sign@pm1}}]]; \[IndentingNewLine] (*\ 
          do\ work\ easy\ w/o\ signs\ *) \[IndentingNewLine]sm1 = Sign@pm1; 
        sm2 = Sign@pm2; \[IndentingNewLine]If[Abs@pm2 > Abs@pm1, 
          m2more = True; m1 = Abs@pm2; m2 = Abs@pm1, m2more = False; 
          m1 = Abs@pm1; m2 = Abs@pm2]; \[IndentingNewLine]v = 
          euclalglist[m2, m1]; 
        gcd = \((Last@v)\)[\([2]\)]; \[IndentingNewLine]v = v[\([All, 3]\)]; 
        v = Reverse@v; 
        v = {0}~Join~v; \[IndentingNewLine]v = 
          euclbacksolve[1, \(-v[\([3]\)]\), 3, v]; \[IndentingNewLine] (*\ 
          replace\ signs . \ see\ note\ *) \[IndentingNewLine]If[
          ListQ@v && NumberQ@\(First@v\), \[IndentingNewLine]If[
            m1 === m2, \[IndentingNewLine]v = Abs@v {sm1, \(-sm2\)}; 
            v = Reverse@v\[IndentingNewLine], 
            If[m2more, v = Reverse@v]; \[IndentingNewLine]If[
              sm1 \[Equal] \(-1\), v[\([2]\)] *= \(-1\)]; 
            If[sm2 \[Equal] \(-1\), 
              v[\([1]\)] *= \(-1\)];\[IndentingNewLine]]]; \
\[IndentingNewLine]Return@{gcd, v}\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
luckily mm has convention; +gcd(a,b) = sa + tb /;gcd>0  (lucky \
since matching signs is tedious i'm sure).  for m1==m2 just cp what mm does\
\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["showFactors", "Subsubsection"],

Cell["\<\
12 can be expressed uniquely two ways: a sum of terms 10+2 which is \
\"understood\", and also a product of primes 1*2*2*3\
\>", "Text"],

Cell[BoxData[
    \(showFactors[ai_] := 
      Module[{a, b, x, y, i, c, p, r, v, 
          af}, \[IndentingNewLine]v = {{1, 1}}; af = ai; r = 1; b = 0; i = 1; 
        c = a + 1; \[IndentingNewLine]If[r \[Equal] a, Return[v]]; 
        v = FactorInteger@ai; \[IndentingNewLine]If[ai \[GreaterEqual] 1, 
          PrependTo[
            v, {1, 1}]]; \
\[IndentingNewLine]Print@\[IndentingNewLine]\(DisplayForm@\(RowBox@{Sequence @@ \
\((v /. {x_Integer, y_Integer} \[Rule] 
                        SuperscriptBox[x, 
                          y])\)}\)\); \
\[IndentingNewLine]v\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["changeBaseArr", "Subsubsection"],

Cell[BoxData[
    \(changeBaseArr[np_, bp_] := 
      Module[{n = np, b = bp, top = 1, arr = {}}, \[IndentingNewLine]While[
          n \[GreaterEqual] b, AppendTo[arr, Mod[n, b]]; n = n/b; 
          n = IntegerPart[n]; 
          If[\(++top\) > 100000, Return[{}]]]; \[IndentingNewLine]AppendTo[
          arr, n]; \[IndentingNewLine]Reverse[
          arr]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["caesar  p123", "Subsubsection"],

Cell["note use of 255 not 26 since CharacterCode is ascii", "Text"],

Cell[BoxData[
    \(caesar[str_, enc_:  True] := 
      Block[{s = str, v}, \[IndentingNewLine]f[p_] := 
          Mod[\((p + 3)\), 255]; \[IndentingNewLine]fd[p_] := 
          Mod[\((p - 3)\), 255]; \[IndentingNewLine]If[
          enc, \[IndentingNewLine] (*\ \(Print["\<s=\>", 
                s];\)\ *) \[IndentingNewLine]v = 
            ToCharacterCode@s; \[IndentingNewLine]s = 
            FromCharacterCode@
              f[v]; \[IndentingNewLine] (*\ \(Print["\<crypt=\>", 
                s];\)\ *) \[IndentingNewLine]s\[IndentingNewLine], \
\[IndentingNewLine]v = ToCharacterCode@s; \[IndentingNewLine]s = 
            FromCharacterCode@fd[v]; \[IndentingNewLine] (*\ 
            Print["\<s=\>", 
              s]\ *) \[IndentingNewLine]s\[IndentingNewLine]]\
\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["congQ  pg 120", "Subsubsection"],

Cell[BoxData[
    \(\(k =. ;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(SetAttributes[congQ, Listable]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(SetAttributes[congQM, Listable]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[342]:="],

Cell["\<\
book says for m|(a-b) m Z+ , but the following denies it, shows \
sign follows m across a,b\
\>", "Text"],

Cell["\<\
{Mod[2,4],Mod[-2,4],Mod[-6,4],Mod[6,4],Mod[2,-4],Mod[-2,-4],Mod[-6,-\
4],Mod[6,-4]}\
\>", "Text",
  CellTags->"(lic2) In[345]:="],

Cell["{2,2,2,2,-2,-2,-2,-2}", "Text",
  CellTags->"(lic2) Out[345]="],

Cell["\<\
also, the case m1 != m2  doesn't matter since it's not related to \
cong either way
technically try Mod1 since we do not know a,b,m came from Floor not \
IntegerPart
(it's commented out to make things \"easier\")\
\>", "Text"],

Cell[BoxData[
    \(congQM[a_, b_, m_] := Mod[a, m] \[Equal] Mod[b, m]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[346]:="],

Cell[BoxData[
    \(congQM1[a_, b_, m_] := Mod1[a, m] \[Equal] Mod1[b, m]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[347]:="],

Cell[BoxData[
    \(congQMboth[a_, b_, m_] := 
      Mod[a, m] \[Equal] Mod[b, m] || 
        Mod1[a, m] \[Equal] Mod1[b, m]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[348]:="],

Cell["m|(a-b) or a==b+km same test", "Text"],

Cell[BoxData[
    \(congQ[a_, b_, m_] := 
      Block[{r}, r = \((\((a - b)\)/m)\); 
          IntegerQ@r] /; {a, b, m} \[Element] Integers && 
          m \[NotEqual] 0\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[349]:="],

Cell["\<\
NameQ show only k is left, thus m | (a-b) result was integer
\tor, if dealing with fractions and r is integer, m divided
\tunfortunately if N[] is involved there's maybe no way of checking if m \
divides (a-b)\
\>", "Text"],

Cell[BoxData[
    \(congQ[a_, b_, m_] := 
      Block[{r}, r = \((\((a - b)\)/m)\); 
          NameQ@\((ToString@r)\) || r \[Equal] 0 || IntegerQ@r] /; 
        m \[NotEqual] 0\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[350]:="],

Cell["\<\
congQ[a_,b_,m_]:=(Print@\"forced\";congQ[Rationalize@a,Rationalize@\
b,Rationalize@m])/;m\[NotEqual]0&&(m\[NotElement]Integers)\
\>", "Text"],

Cell[BoxData[
    \(k =. \)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[354]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
congX , congA ,  3 x \[Congruent] Mod[4,7] , find x  p140\
\>", \
"Subsubsection"],

Cell["tested in section: function tests", "Text"],

Cell[BoxData[
    \(\(k =. ;\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[356]:="],

Cell["\<\
note.  as per mm book's explination: 0 x in eq. are replaced by 0 \
(ie, 0 Mod[4,0]==0) (evaluation reduces expr as soon as possible and 0's are \
special for time sake).  if these functions return unevaluated * 0 this 0 \
will be sometimes WRONG if you them inside a cong expression (if return not \
checked)\
\>", "Text"],

Cell["\<\
\tm!=0 as congQ won't test it and if there is not div there can be \
no remainder to compare\
\>", "Text"],

Cell[BoxData[
    \(congX[b_, a_, m_, k_:  Symbol["Global`k"]] := 
      If[a \[Equal] 0, 
          0, \((\((b + k\ m)\)\/a)\)] /; {a, b, m} \[Element] Integers && 
          m > 1 && GCD[b, m] \[Equal] 
            1 && \[Not] \((b \[Equal] 0 && GCM[a, m] \[Equal] 1)\)\)], "Text"],

Cell[BoxData[
    \(congX[b_, a_, m_, 
        k_Integer] := \(If[a \[Equal] 0, 0, \((\((b + k\ m)\)\/a)\)] /; 
          a \[NotEqual] 0 && 
            m \[NotEqual] 
              0 && \[Not] \((b \[Equal] 0 && 
                  GCD[Rationalize@a, Rationalize@m] \[Equal] 1)\)\) /; 
        k \[Element] Integers\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[359]:="],

Cell[BoxData[
    \(congX[b_, a_, m_, k_Symbol] := 
      If[a \[Equal] 0, 0, \((\((b + k\ m)\)\/a)\)] /; 
        a \[NotEqual] 0 && 
          m \[NotEqual] 
            0 && \[Not] \((b \[Equal] 0 && 
                GCD[Rationalize@a, Rationalize@m] \[Equal] 1)\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[360]:="],

Cell[BoxData[
    \(congX[b_, a_, 
        m_] := \((Symbol["\<cr`k\>"]; 
          If[a \[Equal] 0, 0, \((\((b + cr`k\ m)\)\/a)\)])\) /; 
        a \[NotEqual] 0 && 
          m \[NotEqual] 
            0 && \[Not] \((b \[Equal] 0 && 
                GCD[Rationalize@a, Rationalize@m] \[Equal] 1)\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[361]:="],

Cell["\<\
congA[b_,m_,k_:Symbol[\"Global`k\"]]:=((b+k \
m))/;{a,b,m}\[Element]Integers&&m>1&&GCD[b,m]\[Equal]1\
\>", "Text"],

Cell[BoxData[
    \(congA[b_, m_, k_Integer] := \((\((b + k\ m)\))\) /; 
        k \[Element] Integers && m \[NotEqual] 0\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[362]:="],

Cell[BoxData[
    \(congA[b_, m_, k_Symbol] := \((\((b + k\ m)\))\) && 
        m \[NotEqual] 0\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[363]:="],

Cell[BoxData[
    \(congA[b_, m_] := \((Symbol["\<cr`k\>"]; \((b + cr`k\ m)\))\) && 
        m \[NotEqual] 0\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[364]:="],

Cell[BoxData[
    \(\(k =. ;\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[374]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["crbruteforce , crtestnotcong , crtestcong", "Subsubsection"],

Cell[BoxData[
    \(\(\( (*\ 
      find\ n\ cong\ brute\ force\ search\ \
*) \)\(\[IndentingNewLine]\)\(crbruteforce[bi_, mi_, n_:  3, x_:  0, 
        lim_:  5000] := \[IndentingNewLine]Module[{i, j, v, hit, c}, v = {}; 
        c = 0; \[IndentingNewLine]Do[
          hit = False; \[IndentingNewLine]Do[\(If[
                True =!= congQ[j + x, bi[\([i]\)], 
                    mi[\([i]\)]], \[IndentingNewLine]hit = 
                  True];\)\[IndentingNewLine], {i, 
              Length@mi}]; \[IndentingNewLine]If[\[Not] hit, \(++c\); 
            v = Append[v, j + x]]; \[IndentingNewLine]If[c \[GreaterEqual] n, 
            Return[v]];\[IndentingNewLine], {j, 0, lim}\[IndentingNewLine]]; 
        v]\)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      show\ any\ ai\ cong\ to\ bi\ mod\ mi\ \
*) \)\(\[IndentingNewLine]\)\(crtestcong[bi_, mi_, ai_, 
        dbg_:  False] := \[IndentingNewLine]Module[{i, j, a, v, hit}, 
        v = {}; \[IndentingNewLine]If[Head[ai] =!= List, a = {ai}, 
          a = ai]; \[IndentingNewLine]Do[
          hit = False; \[IndentingNewLine]Do[\(If[
                congQ[a[\([j]\)], bi[\([i]\)], 
                  mi[\([i]\)]], \[IndentingNewLine]\(If[dbg, 
                    v = Append[
                        v, {a[\([j]\)], bi[\([i]\)], 
                          mi[\([i]\)]}]];\)\[IndentingNewLine], \(hit = 
                    True;\)];\)\[IndentingNewLine], {i, 
              Length@mi}]; \[IndentingNewLine]If[\[Not] dbg && \[Not] hit, 
            v = Append[v, a[\([j]\)]]];\[IndentingNewLine], {j, 
            Length@a}]; \[IndentingNewLine]v]\)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      show\ any\ ai\ not\ cong\ to\ bi\ mod\ mi\ *) \)\(\[IndentingNewLine]\)\
\(crtestnotcong[bi_, mi_, ai_, 
        dbg_:  False] := \[IndentingNewLine]Module[{i, j, a, v, hit}, 
        v = {}; \[IndentingNewLine]If[Head[ai] =!= List, a = {ai}, 
          a = ai]; \[IndentingNewLine]Do[
          hit = False; \[IndentingNewLine]Do[\(If[
                True =!= congQ[a[\([j]\)], bi[\([i]\)], 
                    mi[\([i]\)]], \[IndentingNewLine]If[dbg, 
                  v = Append[
                      v, {a[\([j]\)], bi[\([i]\)], 
                        mi[\([i]\)]}]]; \[IndentingNewLine]hit = 
                  True;];\)\[IndentingNewLine], {i, 
              Length@mi}]; \[IndentingNewLine]If[\[Not] dbg && hit, 
            v = Append[v, a[\([j]\)]]];\[IndentingNewLine], {j, 
            Length@a}]; \[IndentingNewLine]v]\)\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["inverseMod  by euclextgcd", "Subsubsection"],

Cell[BoxData[
    \(SetAttributes[inverseMod, {Listable, NumericFunction}]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[388]:="],

Cell[BoxData[
    \(inverseMod[b_, \(-1\), m_] := 
      inverseMod[b, m] /; Abs@m >= 1 && GCD[b, m] \[Equal] 1\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[390]:="],

Cell[BoxData[
    \(inverseMod[pm2_Integer, 
        pm1_Integer] := \[IndentingNewLine]Module[{i, j, m2, m1, sm2, sm1, 
            v}, \[IndentingNewLine] (*\ 
            exclude\ 0' s\ and\ 1' s\ sol\ *) \[IndentingNewLine]v = {}; 
          If[pm1 \[Equal] 0, Return[]]; \[IndentingNewLine]sm1 = Sign@pm1; 
          sm2 = Sign@pm2; \[IndentingNewLine]If[pm1 \[Equal] sm1, 
            If[pm2 === 1, v = 1, v = 0]]; \[IndentingNewLine]If[
            pm2 \[Equal] 0, If[pm1 \[NotEqual] sm1, Return[]]; 
            v = 0]; \[IndentingNewLine]If[pm2 \[Equal] sm2 && v == {}, 
            If[sm2 === \(-1\) && sm1 === 1, v = pm1 - 1, 
              v = sm2]]; \[IndentingNewLine]If[v =!= {}, 
            Return@v]; \[IndentingNewLine] (*\ 
            want\ m2\ a\  - \ m1\ b\  \[Equal] \ 1\ not\ sa + tb \[Equal] 
              1\ *) \[IndentingNewLine]v = 
            euclextgcd[pm2, \(-pm1\)]; \[IndentingNewLine]If[
            ListQ@v, {i, j} = v[\([2]\)]; \[IndentingNewLine] (*\ 
              see\ note\ *) \[IndentingNewLine]If[
              Abs@\((pm1 - i)\) + 
                  Abs@\((pm2 - j)\) > \[IndentingNewLine]Abs@\((pm1 + i)\) + 
                  Abs@\((pm2 + j)\), i = \(-i\); 
              j = \(-j\)]; \[IndentingNewLine]If[i\ pm2 - j\ pm1 != 1, 
              i = pm1 - i; j = pm2 - j]; \[IndentingNewLine]Return@
              i]; \[IndentingNewLine]Print@"\<exhausted\>";\[IndentingNewLine]\
] /; Abs@pm1 >= 1 && GCD[pm1, pm2] \[Equal] 1\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[391]:="],

Cell["\<\
note: soln is not unique.  euclextgcd returns a lowest answer for \
sa-tb==1.  however the other side (b==1->b==-1 using {m1-i,m2-j}) isn't \
insured so.  after correction may be b==1 side.  PowerMod insures the other \
side is the lesser of two choices.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["creqs  define", "Subsubsection"],

Cell[BoxData[
    \(crchart3[bi_, mi_, xi_:  0, xf_:  32, 
        show_:  True] := \[IndentingNewLine]Module[{i, len, gs}, 
        len = Length@mi; \[IndentingNewLine]Do[
          g[i] = Plot[
              Mod[b, mi[\([i]\)]] - Mod[bi[\([i]\)], mi[\([i]\)]], {b, xi, 
                xf}, PlotPoints \[Rule] 1000, 
              DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {RGBColor[Random[], Random[], 
                    Random[]]}]\[IndentingNewLine], {i, 
            len}]; \[IndentingNewLine]gs = 
          Show[Table[g[i], {i, len}], 
            DisplayFunction \[Rule] 
              Identity]; \[IndentingNewLine]gs[\([2]\)] = 
          gs[\([2]\)] /. 
            Rule[DisplayFunction, Identity] \[Rule] 
              Rule[DisplayFunction, $DisplayFunction]; \[IndentingNewLine]If[
          show, Show[gs], gs]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(combpairs[pairs_, 
        stack_] := \((\[IndentingNewLine]If[pairs \[Equal] {}, 
          Return[{stack}]]; \[IndentingNewLine]combpairs[Rest@pairs, 
            Append[stack, \((First@pairs)\)[\([1]\)]]]~
          Join~\[IndentingNewLine]combpairs[Rest@pairs, 
            Append[stack, \((First@
                    pairs)\)[\([2]\)]]]\[IndentingNewLine])\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[401]:="],

Cell[BoxData[
    \(creqs[pbi_, pmi_, 
        eq_:  True] := \[IndentingNewLine]Module[{i, len, bi, mi, r, h, leq, 
          req, sr, eqs, sa}, \[IndentingNewLine]len = 
          Length@pmi; \[IndentingNewLine]If[Times @@ pmi \[Equal] 0, 
          Return[{}]]; \[IndentingNewLine]If[len \[NotEqual] Length@pbi, 
          Message[creqs::"\<lengths\>"]; Return[{}]]; \[IndentingNewLine]bi = 
          pbi; mi = pmi; \[IndentingNewLine]If[len \[Equal] 1, 
          bi = Join[bi, bi]; mi = Join[mi, mi]; 
          len = 2]; \[IndentingNewLine]Do[c[i] = 0; 
          c[i] =. , {i, len}]; \[IndentingNewLine]leq = 
          Reverse@Sort[Table[{mi[\([i]\)], bi[\([i]\)]}, {i, len}], 
              OrderedQ[Abs@{#1, #2}] &]; \[IndentingNewLine]mi = 
          leq[\([All, 1]\)]; \[IndentingNewLine]bi = 
          leq[\([All, 2]\)]; \[IndentingNewLine]r = 
          Mod[bi, mi]; \[IndentingNewLine]h = \((mi - 
              r)\); \[IndentingNewLine] (*\ \(Table[
              If[bi[\([i]\)] \[Equal] 0, 0, 1], {i, 
                len}];\)\ *) \[IndentingNewLine]c =. ; \[IndentingNewLine]If[
          eq === 1 || eq === True, \[IndentingNewLine]req = 
            mi\ Table[c[i], {i, len}]; \ \[IndentingNewLine]leq = 
            req - h; \[IndentingNewLine]sr = 
            Transpose@{leq, req}; \[IndentingNewLine]If[
            eq === 1, \[IndentingNewLine]eqs = 
              sr[\([All, 1]\)]; \[IndentingNewLine]eqs = {Equal @@ 
                  eqs};\[IndentingNewLine]]; \[IndentingNewLine]\(If[
              eq === True, \[IndentingNewLine]eqs = 
                combpairs[
                  sr, {}]; \[IndentingNewLine] (*\ \(sa = {};\)\ \
*) \[IndentingNewLine]eqs = 
                Map[\((Equal @@ #)\) &, 
                  eqs];\[IndentingNewLine]];\)\[IndentingNewLine]]; \
\[IndentingNewLine]{{bi, mi, r, h}, {leq, req, sr, 
            eqs}}\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
legend:
\t(2) mi is tot. width of one saw tooth (above and below x axis) (and first \
R)
\t(3) r==tw-wx is first x axis position of L side of tooth (== mod remainder)
\t(4) mi-r is width==height of tooth above x axis
\t(* minimal build only the L,L,L soln eq *)
\t(* sa=leq/mi;sa={Equal@@sa};eqs=leq;eqs={Equal@@eqs}; *)
\told format was: {bi,mi,h,leq,req,{},r,{,},{},sr,eqs,sa}\
\>", "Text"],

Cell[BoxData[
    \(\(\( (*\ 
      function\ shows\ how\ many\ triangles\ at\ x \[Equal] 
        30\ *) \)\(\[IndentingNewLine]\)\(crxtotriag[mi_, 
        x_] := \((1/mi)\)\ x\)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      function\ shows\ x\ given\ num\ triag\ \
*) \)\(\[IndentingNewLine]\)\(crtriagtox[mi_, x_] := mi\ x\)\)\)], "Input",
  InitializationCell->True],

Cell["\<\
note crxtotriag is special use it is Not a kind of accurate unit \
conversion.  it's close for small nums.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["crfindnSolnAfterxInEqm  define", "Subsubsection"],

Cell[BoxData[
    \(crfindnSolnAfterxInEqm[pbi_, pmi_, n_:  3, xi_:  0, slim_:  False, 
        mp_:  1, modchk_:  False, 
        dbg_:  False] := \[IndentingNewLine]Module[{i, j, len, mi, bi, r, h, 
          x, y, z, ans, eqn, sol, arr, xf, t2, s0, t3, t1, numeqs, mst, mend, 
          xst, xend, dir, m, ints, ok, step, off, trlim, final, cc, 
          ad}, \[IndentingNewLine]i = 0; 
        sol = 0; \[IndentingNewLine]If[
          mp === 1, \[IndentingNewLine]t1 = 
            creqs[pbi, pmi, 1], \[IndentingNewLine]t1 = 
            creqs[pbi, pmi]\[IndentingNewLine]]; \[IndentingNewLine]If[
          Length@t1 \[LessEqual] 1, Return[t1]]; 
        If[Length@t1[\([1]\)] \[LessEqual] 1, 
          Return[t1]]; \[IndentingNewLine]{bi, mi, r, h} = t1[\([1]\)]; 
        t1 = t1[\([2]\)]; \[IndentingNewLine]numeqs = 
          Length@t1[\([4]\)]; \[IndentingNewLine]If[numeqs < 2\ ^len, 
          Return[{}]]; \[IndentingNewLine]len = 
          Length@mi; \[IndentingNewLine] (*\(ad = 
              LCM @@ mi;\)*) \[IndentingNewLine]If[False && mp \[Equal] 1, 
          If[crexclude[bi, mi, r, h, len] \[Equal] False, 
            Return[{}]]]; \[IndentingNewLine]If[
          mp === All, \[IndentingNewLine]mst = 1; 
          mend = numeqs, \[IndentingNewLine]mst = mp; 
          mend = mp; \[IndentingNewLine]If[numeqs < mp, 
            Message[crfindnSolnAfterxInEqm::"\<noeqn\>", mp]; 
            Return[{}]]\[IndentingNewLine]]; \[IndentingNewLine]If[
          slim === False, \[IndentingNewLine]trlim = 
            Abs@\((n\ IntegerPart@\(N@\(crxtotriag[mi, 
                            Times @@ mi]\)[\([1]\)]\))\) + 
              1, \[IndentingNewLine]trlim = 
            slim\[IndentingNewLine]]; \[IndentingNewLine]Do[c[i] = 0; 
          c[i] =. , {i, len}]; \[IndentingNewLine]cc = 
          Table[c[i], {i, len}]; \[IndentingNewLine] (*\ 
          change\ x\ units\ to\ triag\ units\ *) \[IndentingNewLine]t2 = 
          t1[\([4, mst]\)];  (*\(creqs[]\)[\([2, 4, 
              mst]\)]*) \[IndentingNewLine]x = 
          c[1] /. \(Solve[t2[\([1]\)] \[Equal] xi, 
                c[1]]\)[\([1]\)]; \[IndentingNewLine]xf = 
          Floor@x; \[IndentingNewLine]t3 = 
          c[1] /. \(Solve[t2[\([1]\)] \[Equal] xi + 1, 
                c[1]]\)[\([1]\)]; \[IndentingNewLine] (*\ 
          auto\ flip\ if\ it\ will\ run\ other\ way\ *) \[IndentingNewLine]If[
          x > t3, \[IndentingNewLine]xst = xf; xend = xf - trlim; 
          dir = \(-1\);\[IndentingNewLine], xst = xf; xend = xf + trlim; 
          dir = 1;\[IndentingNewLine]]; \[IndentingNewLine]t3 =. ; x =. ; 
        If[xf \[Equal] 0 && 
            xi \[NotEqual] 0, \(++xf\)]; \[IndentingNewLine]If[
          modchk, \[IndentingNewLine]ints = False, 
          ints = IntegerQ /@ mi === 
              Table[True, {i, len}]; \[IndentingNewLine]If[ints, 
            ints = IntegerQ /@ bi === 
                Table[True, {i, 
                    len}]];\[IndentingNewLine]]; \[IndentingNewLine]final = \
{}; arr = {}; \[IndentingNewLine]Do[\[IndentingNewLine] (*\ 
            ok\ to\ use\ t11m . \ 
                convert\ from\ triag\ to\ x\ using\ t2\ then\ check\ soln\ \
was\ on\ axis\ \(creqs[]\)[\([2, 4, m]\)]\ *) \[IndentingNewLine]arr = {}; 
          sol = 0; \[IndentingNewLine]t2 = 
            t1[\([4, m]\)]; \[IndentingNewLine]t3 = 
            Solve[t2, Rest@cc]; \[IndentingNewLine]eqn = 
            Table[c[i], {i, 2, len}] /. 
              t3; \[IndentingNewLine]Do[\[IndentingNewLine]i = 
              id; \[IndentingNewLine]ans = 
              eqn /. c[1] \[Rule] i; \[IndentingNewLine]If[
              Length@ans \[GreaterEqual] 1, ans = ans[\([1]\)], 
              Continue[]]; \[IndentingNewLine]z = \((t2[\([1]\)] /. 
                  c[1] \[Rule] i)\); \[IndentingNewLine]If[
              ints, \[IndentingNewLine]If[
                IntegerQ@z, \[IndentingNewLine]ans = 
                  Join[ans, {z}]; \[IndentingNewLine]If[
                  IntegerQ /@ ans === 
                    Table[True, {j, len}], \[IndentingNewLine]AppendTo[arr, 
                    z]; \(++sol\)\[IndentingNewLine]]]\[IndentingNewLine], \
\[IndentingNewLine]\(If[congQ[z, r, mi] === Table[True, {j, len}], 
                  AppendTo[arr, 
                    z]; \(++sol\)];\)\[IndentingNewLine]]; \
\[IndentingNewLine]If[sol \[GreaterEqual] n, 
              Break[]];\[IndentingNewLine], {id, xst, xend, 
              dir}\[IndentingNewLine]]; \[IndentingNewLine]If[
            arr \[NotEqual] {}, 
            AppendTo[final, arr]];\[IndentingNewLine], {m, mst, 
            mend}]; \[IndentingNewLine]If[sol < n, 
          Message[crfindnSolnAfterxInEqm::"\<exhausted\>"]]; \
\[IndentingNewLine]If[Length@final \[Equal] 1, arr, 
          final]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
quirk: neg bi,mi can run opposite direction of subst so this \
\"fixes\".  it's very ornary to fix if it doesn't work remove all fixes do \
after exit.  note trlim is loop limit not search boundry.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["cralign1", "Subsubsection"],

Cell["see align1", "Text"],

Cell["\<\
cralign1 doesn't know if we align all Rs or all Ls of sawtooth \
triangles (how data is supplied). crtwo uses all Ls aligned.\
\>", "Text"],

Cell[BoxData[
    \(cralign1[m1_, o1_, m2_, bothoff_:  0] := 
      Module[{i, j, gcd, x}, \[IndentingNewLine]If[Mod[o1, m2] \[Equal] 0, 
          Return[o1 + bothoff]]; \[IndentingNewLine]gcd = 
          GCD[m1, m2]; \[IndentingNewLine]If[
          gcd === 1, \[IndentingNewLine]j = 
            PowerMod[\(-m1\), \(-1\), m2]; \[IndentingNewLine]If[
            j \[NotEqual] 0, j = Mod[o1\ j, m2]]; \[IndentingNewLine]Return[
            j\ m1 + o1 + bothoff]\[IndentingNewLine], \[IndentingNewLine]\(If[
              Mod[o1, gcd] \[Equal] 0, \[IndentingNewLine]x = 
                m2/gcd; \[IndentingNewLine]j = 
                PowerMod[\(-m1\)/gcd, \(-1\), x]; \[IndentingNewLine]If[
                j \[NotEqual] 0, 
                j = Mod[\((o1/gcd)\)\ j, m2/gcd]]; \[IndentingNewLine]Return[
                j\ m1 + o1 + 
                  bothoff]\[IndentingNewLine]];\)\[IndentingNewLine]];\
\[IndentingNewLine]]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2124]:="],

Cell["\<\
more true: \
If[m2\[LessEqual]o1&&Mod[o1,m2]\[Equal]0,Return[o1+bothoff]]; ... \
If[Mod[o1,m2]===0,j=o1-m2.  this is inner loop of cr: for good timings do the \
least work possible, \"truer values\" do not change cr, so just return \
0.\
\>", "Text"],

Cell["\<\
if o1>m1 do not answer (a wave isn't longer than itself)(or bad, \
o1=Mod[o1,m1]).  however i don't check because: 1) speed critical  2) if \
crtwo isn't broken it won't ask  3) crtwo uses negative offsetting (actually \
just barfs data) and this function doesn't know that and if it did it \
wouldn't be it's job.  4) can't have it both ways it can't both quit but \
still work with negatives, ie...
\t(* If[Abs@o1>Abs@m1,Return[]]; *)\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["align1", "Subsubsection"],

Cell["see \"plots, align1 tab, ...\"", "Text"],

Cell[BoxData[
    \(align1[m1_, o1_, m2_, bothoff_:  0] := 
      Module[{i, j, gcd, x}, \[IndentingNewLine]If[Mod[o1, m2] \[Equal] 0, 
          Return[{o1/m2, 0, o1 + bothoff}]]; \[IndentingNewLine]gcd = 
          GCD[m1, m2]; \[IndentingNewLine]If[
          gcd === 1, \[IndentingNewLine]j = 
            PowerMod[\(-m1\), \(-1\), m2]; \[IndentingNewLine]If[
            j \[NotEqual] 0, 
            j = Mod[o1\ j, m2]]; \[IndentingNewLine]i = \((j\ m1 + 
                o1)\); \[IndentingNewLine]Return[{i/m2, j, 
              i + bothoff}]\[IndentingNewLine], \[IndentingNewLine]\(If[
              Mod[o1, gcd] \[Equal] 0, \[IndentingNewLine]x = 
                m2/gcd; \[IndentingNewLine]j = 
                PowerMod[\(-m1\)/gcd, \(-1\), x]; \[IndentingNewLine]If[
                j \[NotEqual] 0, 
                j = Mod[\((o1/gcd)\)\ j, 
                    m2/gcd]]; \[IndentingNewLine]i = \((j\ m1 + 
                    o1)\); \[IndentingNewLine]Return[{i/m2, j, 
                  i + bothoff}]\[IndentingNewLine]];\)\[IndentingNewLine]];\
\[IndentingNewLine]]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2125]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["cralign1 align1   table", "Subsubsection"],

Cell[BoxData[
    \(testa1[m1_, m2_, pstart_:  True, pend_:  True] := 
      Module[{start, end, i, j, v}, \[IndentingNewLine]start = pstart; 
        end = pend; \[IndentingNewLine]If[pstart, 
          start = \(-Abs@\((m1)\)\) - 2]; \[IndentingNewLine]If[pend, 
          end = Abs@\((2\ m1)\) + 2]; \[IndentingNewLine]Table[
          v = align1[m1, i, m2]; {m2\ , v[\([1]\)], v[\([2]\)], m1, i, 
            v[\([3]\)]}\[IndentingNewLine], {i, start, 
            end}]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[447]:="],

Cell[BoxData[
    \(testcra1[m1_, m2_, pstart_:  True, pend_:  True] := 
      Module[{start, end, i, j, v}, \[IndentingNewLine]start = pstart; 
        end = pend; \[IndentingNewLine]If[pstart, 
          start = \(-Abs@\((m1)\)\) - 2]; \[IndentingNewLine]If[pend, 
          end = Abs@\((2\ m1)\) + 2]; \[IndentingNewLine]Table[{m2\ , m1, i, 
            cralign1[m1, i, m2]}\[IndentingNewLine], {i, start, 
            end}]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[449]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["crtwo", "Subsubsection"],

Cell["\ta little longer, incl. for redundancy", "Text"],

Cell[BoxData[
    \(crtwo2[bi_, mi_, dbg_:  False] := 
      Module[{i, j, len, r, h, a4, ad, b1, b2, m1, m2, r1, 
          r2}, \[IndentingNewLine]len = Length@mi; \[IndentingNewLine]If[
          len \[NotEqual] Length@bi, Message[crtwo::"\<lengths\>"]; 
          Return[{}]]; \[IndentingNewLine]m1 = mi[\([1]\)]; 
        If[m1 \[Equal] 0, Return[{Null, Null}]]; \[IndentingNewLine]b1 = 
          bi[\([1]\)]; r1 = Abs@Mod[b1, Abs@m1]; 
        i = 0; \[IndentingNewLine]If[len \[Equal] 1, 
          Return@crtwo[{b1, b1}, {m1, m1}, 
              dbg]]; \[IndentingNewLine]Do[\[IndentingNewLine]m2 = 
            mi[\([j]\)]; 
          If[m2 \[Equal] 0, b1 = Null; m1 = Null; 
            Break[]]; \[IndentingNewLine]r2 = 
            Abs@Mod[bi[\([j]\)], Abs@m2]; \[IndentingNewLine]If[
            dbg, \(++i\); \[IndentingNewLine]Print@{"\<obj \>", 
                LCM[m1, m2], {Abs@m1, r1 - r2, Abs@m2, r2}, {b1, 
                  bi[\([j]\)]}, {m1, m2}, {r1, r2}, 
                i}; \[IndentingNewLine]Print@{"\<cr2 cr\>", \
\[IndentingNewLine]If[r1 > r2, 
                  b1 = cralign1[Abs@m1, r1 - r2, Abs@m2, 
                      r2], \[IndentingNewLine]b1 = 
                    cralign1[Abs@m2, r2 - r1, Abs@m1, 
                      r1]]\[IndentingNewLine], 
                crtlist[{b1, bi[\([j]\)]}, {m1, m2}, 1, 
                  2]\[IndentingNewLine]}\[IndentingNewLine]]; \
\[IndentingNewLine]If[
            r1 > r2, \[IndentingNewLine]b1 = 
              cralign1[Abs@m1, r1 - r2, Abs@m2, r2], \[IndentingNewLine]b1 = 
              cralign1[Abs@m2, r2 - r1, Abs@m1, 
                r1]\[IndentingNewLine]]; \[IndentingNewLine]If[
            b1 \[Equal] Null, m1 = Null; Break[]]; \[IndentingNewLine]\(m1 = 
            LCM[m1, m2]; 
          r1 = Mod[b1, m1]\), \[IndentingNewLine]{j, 2, 
            len}]; \[IndentingNewLine]{b1, m1}]\)], "Input",
  InitializationCell->True],

Cell["\<\
the trick: the last answer becomes what the next pair must align \
with (if congruence is possible).  (choice of b1+ rather than least abs, is \
arbitrary)\
\>", "Text"],

Cell[BoxData[
    \(crtwo[bi_, mi_, 
        dbg_:  False] := \[IndentingNewLine]Module[{i, j, len, b1, b2, m1, 
          m2, r1, r2}, \[IndentingNewLine]len = 
          Length@mi; \[IndentingNewLine]If[len \[NotEqual] Length@bi, 
          Message[crtwo::"\<lengths\>"]; Return[{}]]; \[IndentingNewLine]m1 = 
          mi[\([1]\)]; 
        If[m1 \[Equal] 0, Return[{Null, Null}]]; \[IndentingNewLine]b1 = 
          bi[\([1]\)]; r1 = Mod[b1, m1]; 
        i = 0; \[IndentingNewLine]If[len \[Equal] 1, Print@{b1, m1}; 
          Return@crtwo[{b1, b1}, {m1, m1}, 
              dbg]]; \[IndentingNewLine]Do[\[IndentingNewLine]m2 = 
            mi[\([j]\)]; 
          If[m2 \[Equal] 0, b1 = Null; m1 = Null; 
            Break[]]; \[IndentingNewLine]r2 = 
            Mod[bi[\([j]\)], m2]; \[IndentingNewLine]If[
            dbg, \(++i\); \[IndentingNewLine]Print@{"\<obj \>", 
                LCM[m1, m2], {m1, r1 - r2, m2, r2}, {b1, bi[\([j]\)]}, {m1, 
                  m2}, {r1, r2}, i}; \[IndentingNewLine]Print@{"\<cr2 cr\>", 
                cralign1[m1, r1 - r2, m2, r2], 
                crtlist[{b1, bi[\([j]\)]}, {m1, m2}, 1, 
                  2]}\[IndentingNewLine]]; \[IndentingNewLine]b1 = 
            cralign1[m1, r1 - r2, m2, r2]; \[IndentingNewLine]If[
            b1 \[Equal] Null, Break[]]; \[IndentingNewLine]\(m1 = 
            LCM[m1, m2]; 
          r1 = Mod[b1, m1]\), \[IndentingNewLine]{j, 2, 
            len}]; \[IndentingNewLine]If[b1 \[Equal] Null, 
          Return[{Null, Null}]]; \[IndentingNewLine]If[
          b1 < 0, {b1 + m1, m1}, {b1, m1}]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["crAll", "Subsubsection"],

Cell["\<\
theory:
\tsoln repeat every n beginning at m , no matter how many vectors are \
involved\
\>", "Text"],

Cell[BoxData[
    \(crAll[bi_, mi_, n_:  5, x_:  0, lim_:  False, m_:  1] := 
      Module[{r, i, xi, avediff, sign, v}, \[IndentingNewLine]v = 
          Flatten@crfindnSolnAfterxInEqm[bi, mi, 2, x, lim, 
              m]; \[IndentingNewLine]If[Length@v < 2, Return[v]]; 
        avediff = LCM @@ mi; \[IndentingNewLine]r = First@v; 
        xi = Ceiling[\((x - r)\)/avediff]; \[IndentingNewLine]Table[
          r + \((i\ avediff)\), {i, xi, 
            xi + n - 1}]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(crAll2[v_, n_:  5, x_:  0, lim_:  False] := 
      Module[{r, i, xi, avediff, sign}, \[IndentingNewLine]r = First@v; 
        If[r \[Equal] Null, Return[{}]]; 
        avediff = Last@v; \[IndentingNewLine]xi = 
          Ceiling[\((x - r)\)/avediff]; \[IndentingNewLine]Table[
          r + \((i\ avediff)\), {i, xi, 
            xi + n - 1}]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["IntToModTups  AddModTups  TupsModToInt  p142", "Subsubsection"],

Cell[BoxData[
    \(IntToModTups[n_, v_: {99, 98, 97, 95}] := 
      Module[{vp}, \[IndentingNewLine]vp = Times @@ v; 
        If[\[Not] IntegerQ@vp, Return[]]; \[IndentingNewLine]If[n > vp, 
          Return["\<n too large for v\>"]]; \[IndentingNewLine]Map[\((Mod[
                n, #])\) &, v]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(AddModTups[n_, m_, v_: {99, 98, 97, 95}] := 
      Module[{vp, fv}, vp = n + m; vp~Mod~v]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(SubModTups[n_, m_, v_: {99, 98, 97, 95}] := 
      Module[{vp, fv}, vp = n - m; vp~Mod~v]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MulModTups[n_, m_, v_: {99, 98, 97, 95}] := 
      Module[{vp, fv}, vp = n*m; vp~Mod~v]\)], "Input",
  InitializationCell->True],

Cell["\tuse mag as sign, saves effort in tracking separately", "Text"],

Cell[BoxData[
    \(TupsModToInt[n_, v_: {99, 98, 97, 95}, signed_:  True] := 
      Module[{x, z}, x = ChineseRemainder[n, v]; 
        If[\[Not] IntegerQ@x, Return[]]; z = Times @@ v; 
        If[signed && x \[GreaterEqual] z/2, x - z, x]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["end package", "Subsubsection"],

Cell[BoxData[
    \(\(End[];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(On[General::"\<spell\>"]\)\(\ \)\( (*\ 
      turn\ off\ has\ similar\ spelling\ messages\ *) \)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(EndPackage[];\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[270]:="]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowSize->{648, 607},
WindowMargins->{{0, Automatic}, {-11, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "(lic2) In[343]:="->{
    Cell[6540, 188, 599, 9, 155, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[343]:="]},
  "(lic2) In[344]:="->{
    Cell[7142, 199, 369, 6, 91, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[344]:="]},
  "(lic2) In[357]:="->{
    Cell[7514, 207, 382, 6, 91, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[357]:="]},
  "(lic2) In[358]:="->{
    Cell[7899, 215, 409, 7, 107, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[358]:="]},
  "(lic2) In[389]:="->{
    Cell[8311, 224, 325, 7, 61, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[389]:="]},
  "(lic2) In[2719]:="->{
    Cell[8639, 233, 351, 6, 91, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2719]:="]},
  "(lic2) In[405]:="->{
    Cell[8993, 241, 500, 8, 123, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[405]:="]},
  "(lic2) In[406]:="->{
    Cell[9496, 251, 148, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[406]:="]},
  "(lic2) In[2193]:="->{
    Cell[9647, 256, 741, 12, 187, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2193]:="]},
  "(lic2) In[2194]:="->{
    Cell[10391, 270, 169, 4, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2194]:="]},
  "(lic2) In[2195]:="->{
    Cell[10563, 276, 180, 4, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2195]:="]},
  "(lic2) In[2196]:="->{
    Cell[10746, 282, 188, 4, 43, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2196]:="]},
  "(lic2) In[437]:="->{
    Cell[10937, 288, 666, 10, 171, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[437]:="]},
  "(lic2) In[427]:="->{
    Cell[11606, 300, 570, 9, 139, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[427]:="]},
  "(lic2) In[1013]:="->{
    Cell[12179, 311, 961, 14, 235, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[1013]:="]},
  "(lic2) In[1014]:="->{
    Cell[13143, 327, 157, 4, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[1014]:="]},
  "(lic2) In[2186]:="->{
    Cell[13303, 333, 581, 9, 139, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2186]:="]},
  "(lic2) In[342]:="->{
    Cell[26947, 708, 124, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[342]:="]},
  "(lic2) In[345]:="->{
    Cell[27192, 718, 139, 4, 31, "Text",
      CellTags->"(lic2) In[345]:="]},
  "(lic2) Out[345]="->{
    Cell[27334, 724, 69, 1, 31, "Text",
      CellTags->"(lic2) Out[345]="]},
  "(lic2) In[346]:="->{
    Cell[27645, 735, 143, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[346]:="]},
  "(lic2) In[347]:="->{
    Cell[27791, 740, 146, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[347]:="]},
  "(lic2) In[348]:="->{
    Cell[27940, 745, 197, 5, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[348]:="]},
  "(lic2) In[349]:="->{
    Cell[28187, 754, 242, 6, 43, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[349]:="]},
  "(lic2) In[350]:="->{
    Cell[28668, 769, 249, 6, 43, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[350]:="]},
  "(lic2) In[354]:="->{
    Cell[29074, 782, 98, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[354]:="]},
  "(lic2) In[356]:="->{
    Cell[29356, 797, 103, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[356]:="]},
  "(lic2) In[359]:="->{
    Cell[30203, 822, 389, 9, 80, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[359]:="]},
  "(lic2) In[360]:="->{
    Cell[30595, 833, 340, 8, 80, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[360]:="]},
  "(lic2) In[361]:="->{
    Cell[30938, 843, 372, 9, 62, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[361]:="]},
  "(lic2) In[362]:="->{
    Cell[31440, 859, 194, 4, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[362]:="]},
  "(lic2) In[363]:="->{
    Cell[31637, 865, 168, 4, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[363]:="]},
  "(lic2) In[364]:="->{
    Cell[31808, 871, 181, 4, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[364]:="]},
  "(lic2) In[374]:="->{
    Cell[31992, 877, 103, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[374]:="]},
  "(lic2) In[388]:="->{
    Cell[34895, 946, 147, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[388]:="]},
  "(lic2) In[390]:="->{
    Cell[35045, 951, 184, 4, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[390]:="]},
  "(lic2) In[391]:="->{
    Cell[35232, 957, 1544, 26, 315, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[391]:="]},
  "(lic2) In[401]:="->{
    Cell[38052, 1017, 456, 9, 91, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[401]:="]},
  "(lic2) In[2124]:="->{
    Cell[46810, 1203, 1002, 18, 267, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2124]:="]},
  "(lic2) In[2125]:="->{
    Cell[48658, 1248, 1155, 21, 299, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[2125]:="]},
  "(lic2) In[447]:="->{
    Cell[49901, 1276, 567, 10, 139, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[447]:="]},
  "(lic2) In[449]:="->{
    Cell[50471, 1288, 520, 9, 139, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[449]:="]},
  "(lic2) In[270]:="->{
    Cell[57605, 1463, 110, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[270]:="]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"(lic2) In[343]:=", 58356, 1486},
  {"(lic2) In[344]:=", 58492, 1490},
  {"(lic2) In[357]:=", 58627, 1494},
  {"(lic2) In[358]:=", 58762, 1498},
  {"(lic2) In[389]:=", 58898, 1502},
  {"(lic2) In[2719]:=", 59034, 1506},
  {"(lic2) In[405]:=", 59170, 1510},
  {"(lic2) In[406]:=", 59306, 1514},
  {"(lic2) In[2193]:=", 59442, 1518},
  {"(lic2) In[2194]:=", 59581, 1522},
  {"(lic2) In[2195]:=", 59719, 1526},
  {"(lic2) In[2196]:=", 59857, 1530},
  {"(lic2) In[437]:=", 59994, 1534},
  {"(lic2) In[427]:=", 60132, 1538},
  {"(lic2) In[1013]:=", 60270, 1542},
  {"(lic2) In[1014]:=", 60410, 1546},
  {"(lic2) In[2186]:=", 60548, 1550},
  {"(lic2) In[342]:=", 60686, 1554},
  {"(lic2) In[345]:=", 60822, 1558},
  {"(lic2) Out[345]=", 60925, 1561},
  {"(lic2) In[346]:=", 61027, 1564},
  {"(lic2) In[347]:=", 61163, 1568},
  {"(lic2) In[348]:=", 61299, 1572},
  {"(lic2) In[349]:=", 61435, 1576},
  {"(lic2) In[350]:=", 61571, 1580},
  {"(lic2) In[354]:=", 61707, 1584},
  {"(lic2) In[356]:=", 61842, 1588},
  {"(lic2) In[359]:=", 61978, 1592},
  {"(lic2) In[360]:=", 62114, 1596},
  {"(lic2) In[361]:=", 62250, 1600},
  {"(lic2) In[362]:=", 62386, 1604},
  {"(lic2) In[363]:=", 62522, 1608},
  {"(lic2) In[364]:=", 62658, 1612},
  {"(lic2) In[374]:=", 62794, 1616},
  {"(lic2) In[388]:=", 62930, 1620},
  {"(lic2) In[390]:=", 63066, 1624},
  {"(lic2) In[391]:=", 63202, 1628},
  {"(lic2) In[401]:=", 63341, 1632},
  {"(lic2) In[2124]:=", 63479, 1636},
  {"(lic2) In[2125]:=", 63621, 1640},
  {"(lic2) In[447]:=", 63762, 1644},
  {"(lic2) In[449]:=", 63901, 1648},
  {"(lic2) In[270]:=", 64039, 1652}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 41, 0, 31, "Text"],
Cell[1761, 51, 37, 0, 31, "Text"],
Cell[1801, 53, 41, 0, 31, "Text"],
Cell[1845, 55, 26, 0, 31, "Text"],
Cell[1874, 57, 44, 0, 31, "Text"],
Cell[1921, 59, 66, 0, 31, "Text"],
Cell[1990, 61, 56, 0, 31, "Text"],
Cell[2049, 63, 386, 9, 150, "Text"],
Cell[2438, 74, 35, 0, 31, "Text"],
Cell[2476, 76, 57, 0, 31, "Text"],
Cell[2536, 78, 913, 24, 405, "Text"],

Cell[CellGroupData[{
Cell[3474, 106, 30, 0, 42, "Subsubsection"],
Cell[3507, 108, 596, 11, 171, "Input",
  InitializationCell->True],
Cell[4106, 121, 114, 2, 27, "Input",
  InitializationCell->True],
Cell[4223, 125, 136, 3, 27, "Input",
  InitializationCell->True],
Cell[4362, 130, 164, 3, 27, "Input",
  InitializationCell->True],
Cell[4529, 135, 745, 11, 315, "Input",
  InitializationCell->True],
Cell[5277, 148, 159, 3, 43, "Input",
  InitializationCell->True],
Cell[5439, 153, 114, 2, 27, "Input",
  InitializationCell->True],
Cell[5556, 157, 127, 3, 27, "Input",
  InitializationCell->True],
Cell[5686, 162, 206, 4, 59, "Input",
  InitializationCell->True],
Cell[5895, 168, 141, 3, 27, "Input",
  InitializationCell->True],
Cell[6039, 173, 143, 3, 27, "Input",
  InitializationCell->True],
Cell[6185, 178, 189, 3, 59, "Input",
  InitializationCell->True],
Cell[6377, 183, 160, 3, 43, "Input",
  InitializationCell->True],
Cell[6540, 188, 599, 9, 155, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[343]:="],
Cell[7142, 199, 369, 6, 91, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[344]:="],
Cell[7514, 207, 382, 6, 91, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[357]:="],
Cell[7899, 215, 409, 7, 107, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[358]:="],
Cell[8311, 224, 325, 7, 61, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[389]:="],
Cell[8639, 233, 351, 6, 91, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2719]:="],
Cell[8993, 241, 500, 8, 123, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[405]:="],
Cell[9496, 251, 148, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[406]:="],
Cell[9647, 256, 741, 12, 187, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2193]:="],
Cell[10391, 270, 169, 4, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2194]:="],
Cell[10563, 276, 180, 4, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2195]:="],
Cell[10746, 282, 188, 4, 43, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2196]:="],
Cell[10937, 288, 666, 10, 171, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[437]:="],
Cell[11606, 300, 570, 9, 139, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[427]:="],
Cell[12179, 311, 961, 14, 235, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[1013]:="],
Cell[13143, 327, 157, 4, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[1014]:="],
Cell[13303, 333, 581, 9, 139, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2186]:="],
Cell[13887, 344, 430, 6, 123, "Input",
  InitializationCell->True],
Cell[14320, 352, 202, 4, 43, "Input",
  InitializationCell->True],
Cell[14525, 358, 72, 1, 27, "Input"],
Cell[14600, 361, 50, 0, 31, "Text"],
Cell[14653, 363, 196, 5, 65, "Text"],
Cell[14852, 370, 73, 1, 27, "Input"],
Cell[14928, 373, 135, 3, 27, "Input",
  InitializationCell->True],
Cell[15066, 378, 88, 2, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[15191, 385, 32, 0, 42, "Subsubsection"],
Cell[15226, 387, 51, 0, 31, "Text"],
Cell[15280, 389, 100, 2, 27, "Input",
  InitializationCell->True],
Cell[15383, 393, 830, 17, 203, "Input",
  InitializationCell->True],
Cell[16216, 412, 1044, 21, 267, "Input",
  InitializationCell->True],
Cell[17263, 435, 1054, 21, 267, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[18354, 461, 34, 0, 42, "Subsubsection"],
Cell[18391, 463, 316, 6, 99, "Text"],
Cell[18710, 471, 100, 2, 27, "Input",
  InitializationCell->True],
Cell[18813, 475, 194, 4, 43, "Input",
  InitializationCell->True],
Cell[19010, 481, 70, 0, 31, "Text"],
Cell[19083, 483, 131, 3, 27, "Input",
  InitializationCell->True],
Cell[19217, 488, 95, 2, 41, "Input",
  InitializationCell->True],
Cell[19315, 492, 196, 5, 43, "Input",
  InitializationCell->True],
Cell[19514, 499, 98, 2, 27, "Input",
  InitializationCell->True],
Cell[19615, 503, 98, 2, 27, "Input",
  InitializationCell->True],
Cell[19716, 507, 114, 2, 27, "Input",
  InitializationCell->True],
Cell[19833, 511, 176, 4, 48, "Text"],
Cell[20012, 517, 798, 14, 187, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[20847, 536, 73, 0, 42, "Subsubsection"],
Cell[20923, 538, 60, 0, 31, "Text"],
Cell[20986, 540, 136, 5, 82, "Text"],
Cell[21125, 547, 106, 3, 31, "Text"],
Cell[21234, 552, 290, 6, 59, "Input",
  InitializationCell->True],
Cell[21527, 560, 190, 4, 43, "Input",
  InitializationCell->True],
Cell[21720, 566, 88, 3, 31, "Text"],
Cell[21811, 571, 485, 9, 107, "Input",
  InitializationCell->True],
Cell[22299, 582, 308, 6, 75, "Input",
  InitializationCell->True],
Cell[22610, 590, 135, 5, 82, "Text"],
Cell[22748, 597, 1440, 25, 331, "Input",
  InitializationCell->True],
Cell[24191, 624, 166, 4, 48, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24394, 633, 36, 0, 42, "Subsubsection"],
Cell[24433, 635, 146, 3, 48, "Text"],
Cell[24582, 640, 639, 13, 155, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[25258, 658, 38, 0, 42, "Subsubsection"],
Cell[25299, 660, 420, 8, 123, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[25756, 673, 37, 0, 42, "Subsubsection"],
Cell[25796, 675, 67, 0, 31, "Text"],
Cell[25866, 677, 835, 16, 267, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[26738, 698, 38, 0, 42, "Subsubsection"],
Cell[26779, 700, 71, 2, 27, "Input",
  InitializationCell->True],
Cell[26853, 704, 91, 2, 27, "Input",
  InitializationCell->True],
Cell[26947, 708, 124, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[342]:="],
Cell[27074, 713, 115, 3, 48, "Text"],
Cell[27192, 718, 139, 4, 31, "Text",
  CellTags->"(lic2) In[345]:="],
Cell[27334, 724, 69, 1, 31, "Text",
  CellTags->"(lic2) Out[345]="],
Cell[27406, 727, 236, 6, 65, "Text"],
Cell[27645, 735, 143, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[346]:="],
Cell[27791, 740, 146, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[347]:="],
Cell[27940, 745, 197, 5, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[348]:="],
Cell[28140, 752, 44, 0, 31, "Text"],
Cell[28187, 754, 242, 6, 43, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[349]:="],
Cell[28432, 762, 233, 5, 82, "Text"],
Cell[28668, 769, 249, 6, 43, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[350]:="],
Cell[28920, 777, 151, 3, 48, "Text"],
Cell[29074, 782, 98, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[354]:="]
}, Open  ]],

Cell[CellGroupData[{
Cell[29209, 790, 92, 3, 42, "Subsubsection"],
Cell[29304, 795, 49, 0, 31, "Text"],
Cell[29356, 797, 103, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[356]:="],
Cell[29462, 802, 333, 6, 82, "Text"],
Cell[29798, 810, 116, 3, 48, "Text"],
Cell[29917, 815, 283, 5, 64, "Text"],
Cell[30203, 822, 389, 9, 80, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[359]:="],
Cell[30595, 833, 340, 8, 80, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[360]:="],
Cell[30938, 843, 372, 9, 62, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[361]:="],
Cell[31313, 854, 124, 3, 31, "Text"],
Cell[31440, 859, 194, 4, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[362]:="],
Cell[31637, 865, 168, 4, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[363]:="],
Cell[31808, 871, 181, 4, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[364]:="],
Cell[31992, 877, 103, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[374]:="]
}, Open  ]],

Cell[CellGroupData[{
Cell[32132, 885, 66, 0, 42, "Subsubsection"],
Cell[32201, 887, 759, 14, 187, "Input",
  InitializationCell->True],
Cell[32963, 903, 922, 17, 203, "Input",
  InitializationCell->True],
Cell[33888, 922, 917, 17, 203, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[34842, 944, 50, 0, 42, "Subsubsection"],
Cell[34895, 946, 147, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[388]:="],
Cell[35045, 951, 184, 4, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[390]:="],
Cell[35232, 957, 1544, 26, 315, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[391]:="],
Cell[36779, 985, 278, 5, 82, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37094, 995, 38, 0, 42, "Subsubsection"],
Cell[37135, 997, 914, 18, 203, "Input",
  InitializationCell->True],
Cell[38052, 1017, 456, 9, 91, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[401]:="],
Cell[38511, 1028, 1933, 34, 523, "Input",
  InitializationCell->True],
Cell[40447, 1064, 402, 9, 133, "Text"],
Cell[40852, 1075, 223, 5, 43, "Input",
  InitializationCell->True],
Cell[41078, 1082, 180, 4, 43, "Input",
  InitializationCell->True],
Cell[41261, 1088, 130, 3, 48, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41428, 1096, 55, 0, 42, "Subsubsection"],
Cell[41486, 1098, 4846, 85, 1179, "Input",
  InitializationCell->True],
Cell[46335, 1185, 221, 4, 65, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46593, 1194, 33, 0, 42, "Subsubsection"],
Cell[46629, 1196, 26, 0, 31, "Text"],
Cell[46658, 1198, 149, 3, 48, "Text"],
Cell[46810, 1203, 1002, 18, 267, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2124]:="],
Cell[47815, 1223, 260, 6, 65, "Text"],
Cell[48078, 1231, 460, 8, 116, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48575, 1244, 31, 0, 42, "Subsubsection"],
Cell[48609, 1246, 46, 0, 31, "Text"],
Cell[48658, 1248, 1155, 21, 299, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[2125]:="]
}, Open  ]],

Cell[CellGroupData[{
Cell[49850, 1274, 48, 0, 42, "Subsubsection"],
Cell[49901, 1276, 567, 10, 139, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[447]:="],
Cell[50471, 1288, 520, 9, 139, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[449]:="]
}, Open  ]],

Cell[CellGroupData[{
Cell[51028, 1302, 30, 0, 42, "Subsubsection"],
Cell[51061, 1304, 55, 0, 31, "Text"],
Cell[51119, 1306, 1919, 35, 443, "Input",
  InitializationCell->True],
Cell[53041, 1343, 179, 4, 48, "Text"],
Cell[53223, 1349, 1643, 29, 379, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[54903, 1383, 30, 0, 42, "Subsubsection"],
Cell[54936, 1385, 112, 4, 48, "Text"],
Cell[55051, 1391, 530, 9, 123, "Input",
  InitializationCell->True],
Cell[55584, 1402, 423, 8, 107, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[56044, 1415, 69, 0, 42, "Subsubsection"],
Cell[56116, 1417, 358, 6, 91, "Input",
  InitializationCell->True],
Cell[56477, 1425, 150, 3, 27, "Input",
  InitializationCell->True],
Cell[56630, 1430, 150, 3, 27, "Input",
  InitializationCell->True],
Cell[56783, 1435, 148, 3, 27, "Input",
  InitializationCell->True],
Cell[56934, 1440, 70, 0, 31, "Text"],
Cell[57007, 1442, 282, 5, 59, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[57326, 1452, 36, 0, 42, "Subsubsection"],
Cell[57365, 1454, 71, 2, 27, "Input",
  InitializationCell->True],
Cell[57439, 1458, 163, 3, 27, "Input",
  InitializationCell->True],
Cell[57605, 1463, 110, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[270]:="]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

